********************************************************************************
* Conscious Coupling                                                           *
* Data Cleaning: Removing duplicates and fixed incorrect IDs       	   		   *
********************************************************************************


**#  1. Cleaning of Monitoring Data
* -----------------------------------------------------------------------------* 
	use 	   "$data_raw/WB_Couples_Training_Implementation_Monitoring.dta", clear
	
	duplicates tag RespondentID NameCode, gen(dup)
	drop if        RespondentID == . & NameCode == . // 354 empty observations
	sort 	       RespondentID
	tostring       RespondentID, gen(str_RespondentID)
	
	foreach var in ContactedYesNo ConsentedAgreedtopartakeint AttendedBHAYesNo ///
				   ScaleUpTrainingHomesession1 ScaleUpTrainingHomesession2 ///
				   ScaleUpTrainingHomesession3 StartedScaleUpTrainingYesNo ///
				   ScaleUpTrainingShortCourse1 ScaleUpTrainingShortCourse2 ///
				   ScaleUpTrainingShortCourse3 {
		replace `var' = strtrim(`var')
		replace `var' = strupper(`var')
		tab `var'
	}
	
	forvalues i = 1/3 {
		gen Homesession`i' = (ScaleUpTrainingHomesession`i' == "YES")
		gen SUsession`i'   = (ScaleUpTrainingShortCourse`i' == "YES")
	}
	
	* Type 1 - Identical entries for all variables --> dropping spare observations
	duplicates tag  RespondentID - Reasonforrefusal, gen(dup1)
	duplicates drop RespondentID - Reasonforrefusal, force // 1 obs deleted

	* Type 2 - Duplicate IDs but entries vary (e.g. name)
	duplicates tag  RespondentID, gen(dup_RespondentID)
	tab  			dup_RespondentID

	* Type 3 - Duplicate Names but entries vary
	duplicates tag  NameCode, gen(dup_Name)

	drop if    dup_RespondentID >= 0 & ConsentedAgreedtopartakeint == "NO" // dropping non-consent
	drop if    (Homesession1 == 0 & Homesession2 == 0 & Homesession3 == 0 & ///
				SUsession1 == 0 & SUsession2 == 0 & SUsession3 == 0) // dropping never compliers
	drop 	   dup_Name dup_RespondentID 
	
	duplicates tag RespondentID, gen(dup_RespondentID)
	duplicates tag NameCode, gen(dup_Name)
	tab        dup_Name dup_RespondentID 

	* Cases with unique names but duplicate IDs
	duplicates drop if RespondentID == 444884 // all relevant entries are identical
	drop if            str_RespondentID == "799578" & DateContactedDMYEthiopian == "21jul2022"
	// observation contacted on 27oct2021 and updated monitoring data
	duplicates drop    RespondentID if RespondentID == 444884, force 
	
	replace    SUsession1 = 1 if RespondentID == 1054020041 & DateContactedDMYEthiopian == "28/01/2022"
	// updating the entry on 28/01/2022 with entry from 21jul2022
	drop if    RespondentID == 1054020041 & DateContactedDMYEthiopian == "21jul2022"
	
	replace    RespondentID = 5373020918 if NameCode == 296
	// identical ID with another respondent --> manual correction
	replace    RespondentID = 544759 	 if NameCode == 99 // correct ID extracted from BL survey
	replace    RespondentID = 1074060615 if NameCode == 222
	replace    RespondentID = 274427     if NameCode == 267 // correct ID extracted from BL survey
	replace    RespondentID = 617153     if NameCode == 171 // correct ID extracted from BL survey
	replace    RespondentID = 1094010187 if NameCode == 645 // correct ID extracted from BL survey
	
	drop       str_RespondentID
	tostring   RespondentID, gen (str_RespondentID)
	
	drop       dup_Name dup_RespondentID 
	duplicates tag RespondentID, gen(dup_RespondentID)
	duplicates tag NameCode, gen(dup_Name)
	tab        dup_Name dup_RespondentID // still cases with duplicate ID but different name
	replace    RespondentID = 5373020976 if NameCode == 494 // correct ID extracted from BL survey
	
	drop       dup_Name dup_RespondentID 
	duplicates tag RespondentID, gen(dup_RespondentID)
	duplicates tag NameCode, gen(dup_Name)
	tab        dup_Name dup_RespondentID
	drop if    RespondentID == 1184060065 & DateContactedDMYEthiopian == "21jul2022" & NameCode == 127
	// identical entries, dropping spare
	replace    RespondentID = 1133010393 if NameCode == 598
	
	drop       dup_Name dup_RespondentID 
	duplicates tag RespondentID, gen(dup_RespondentID)
	duplicates tag NameCode, gen(dup_Name)
	tab        dup_Name dup_RespondentID
	
	replace    RespondentID =  1072010137 if NameCode == 24
	replace    SUsession1   =  1          if NameCode == 24 & DateContactedDMYEthiopian == "30nov2021"
	drop if    RespondentID == 1072010137 & DateContactedDMYEthiopian == "21jul2022"

	duplicates drop RespondentID NameCode Homesession1 Homesession2 Homesession3 ///
					SUsession1 SUsession2 SUsession3, force // 12 duplicates deleted
	duplicates list NameCode RespondentID str_RespondentID

	replace    SUsession1 = 1 if NameCode == 232 & DateContactedDMYEthiopian == "28/01/2043"
	replace    DateContactedDMYEthiopian = "28/01/2022" if RespondentID == 1061010072 & /// 
			   NameCode == 232 & DateContactedDMYEthiopian == "28/01/2043"
	drop if    DateContactedDMYEthiopian == "21jul2022" & RespondentID==1061010072 & NameCode == 232 

	replace    SUsession1 = 1 if NameCode == 275 & DateContactedDMYEthiopian == "08oct2021"
	drop if    DateContactedDMYEthiopian == "21jul2022" & RespondentID == 485089 & NameCode == 275

	replace    SUsession1 = 1 if NameCode == 357 & DateContactedDMYEthiopian == "08oct2021"
	drop if    DateContactedDMYEthiopian == "21jul2022" & RespondentID == 373946 & NameCode == 357
	drop if    DateContactedDMYEthiopian == "27-Oct-21" & RespondentID == 5394020482 & NameCode == 686
	
	drop       dup_Name dup_RespondentID str_RespondentID
	tostring   RespondentID, gen (str_RespondentID)
	duplicates tag RespondentID, gen(dup_RespondentID)
	duplicates tag NameCode, gen(dup_Name)
	
	replace    RespondentID = 1073020837 if NameCode == 144
	replace    RespondentID = 557335     if NameCode == 126
	replace    RespondentID = 1182010178 if NameCode == 128
	replace    RespondentID = 1103020539 if NameCode == 639
	replace    RespondentID = 1174060231 if NameCode == 398
	replace    RespondentID = 1101030245 if NameCode == 564
	replace    RespondentID = 250325     if NameCode == 262
	replace    RespondentID = 1103020539 if NameCode == 638
	replace    RespondentID = 1183020141 if NameCode == 83
	replace    RespondentID = 1171010772 if NameCode == 692
	replace    RespondentID = 1202010320 if NameCode == 581
	replace    RespondentID = 1153020192 if NameCode == 558
	replace    RespondentID = 1101070196 if NameCode == 356
	replace    RespondentID = 562404     if NameCode == 498
	replace    RespondentID = 1204061294 if NameCode == 511
	replace    RespondentID = 414891     if NameCode == 65
	replace    RespondentID = 1181020319 if NameCode == 561
	replace    RespondentID = 1154020261 if NameCode == 373
	replace    RespondentID = 1071010264 if NameCode == 332
	replace    RespondentID = 1071030957 if NameCode == 226
	replace    RespondentID = 1204171198 if NameCode == 56
	replace    RespondentID = 1124130333 if NameCode == 364
	replace    RespondentID = 1204131791 if NameCode == 160
	replace    RespondentID = 549908     if NameCode == 216
	replace    RespondentID = 1191030328 if NameCode == 617
	replace    RespondentID = 1151070233 if NameCode == 642
	replace    RespondentID = 1133021840 if NameCode == 417
	replace    RespondentID = 1101030414 if NameCode == 82
	replace    RespondentID = 1174011169 if NameCode == 36
	replace    RespondentID = 1131071285 if NameCode == 680
	replace    RespondentID = 1144021286 if NameCode == 292
	replace    RespondentID = 1176010402 if NameCode == 243
	replace    RespondentID = 1161040150 if NameCode == 378
	replace    RespondentID = 1174060609 if NameCode == 631
	replace    RespondentID = 1111070543 if NameCode == 482

	drop       dup_Name dup_RespondentID str_RespondentID
	tostring   RespondentID,     gen(str_RespondentID)
	duplicates tag RespondentID, gen(dup_RespondentID)
	duplicates tag NameCode,     gen(dup_Name)

	drop if    DateContactedDMYEthiopian == "21jul2022" & RespondentID == 250325
	// updated monitoring data collected in Oct
	drop if    RespondentID == 557335 & NameCode == 126
	drop if    DateContactedDMYEthiopian == "21jul2022" & RespondentID == 549908

	replace    SUsession1 = 1 if NameCode == 83 & DateContactedDMYEthiopian == "10feb2022"
	drop if    DateContactedDMYEthiopian == "21jul2022"  & RespondentID == 1183020141 & NameCode == 83
	drop if    DateContactedDMYEthiopian == "21jul2022"  & RespondentID == 1103020539
	drop if    DateContactedDMYEthiopian == "21jul2022"  & RespondentID == 1154020261
	drop if    DateContactedDMYEthiopian == "21jul2022"  & RespondentID == 1181020319
	drop if    DateContactedDMYEthiopian == "31/01/2022" & RespondentID == 1204061294
	drop if    DateContactedDMYEthiopian == "31/01/2022" & RespondentID == 1204171198

	* Final cleaning
	drop       dup_Name dup_RespondentID str_RespondentID
	tostring   RespondentID,     gen(str_RespondentID)
	duplicates tag RespondentID, gen(dup_RespondentID)
	duplicates tag NameCode,     gen(dup_Name)
	tab        dup_RespondentID dup_Name
	
	duplicates report RespondentID
	duplicates report NameCode // no duplicates left

	egen       n_homesession   = rowtotal(Homesession1 Homesession2 Homesession3)
	egen       n_SUsession     = rowtotal(SUsession1 SUsession2 SUsession3)
	gen        totalsession    = n_homesession + n_SUsession
	gen        complete_HS     = (n_homesession==3)
	gen        complete_SU     = (n_SUsession==3)
	gen        complete_course = (totalsession==6)
	gen        any_session     = (totalsession>0)
	           
	rename     (RespondentID NameCode) (meta_1_1 resp_name_encoded)
	merge      1:1 meta_1_1 using "$data_raw/DOT_Couples_Training_strat_randomization_R2BL.dta", /// 
			   keepusing(meta_1_1) nogen
	merge 	   1:1 meta_1_1 using "$data_raw/DOT_Couples_Training_strat_randomization.dta", /// 
			   keepusing(meta_1_1) nogen
	recode     n_homesession n_SUsession totalsession complete_HS complete_SU ///
			   complete_course any_session (. =0)
	
	drop	   str_RespondentID dup dup1 dup_RespondentID dup_Name
	saveold	   "$data_working/Implementation Monitoring_Consolidated_Report.dta", replace 


**#  2. Cleaning of Screening Data
* -----------------------------------------------------------------------------* 
	use 		 "$data_raw/210328_WB_Couples_Training_Phone_Screening_Final_public.dta", clear
	append using "$data_raw/210728_WB_Couples_Training_Phone_Screening_Final_public.dta", force

	destring 	 dot_1_1, replace
	append using "$data_raw/211206_WB_Couples_Training_Phone_Screening_round2_public.dta", force
	append using "$data_raw/220530_WB_Couples_Training_Phone_Screening_round3_public.dta", force
	drop if 	 dot_1_1 == .  // 1 missing obs

	replace 	 resp_city_pulled = "Bahir Dar" if resp_city_pulled == "Bahir dar"
	replace 	 call_attempt = call_attempt_oth if call_attempt == 0
	recode       consent dot_1_16 (2 =0)

	keep 		 dc_id dot_1_1 resp_phone_code resp_city_pulled call_attempt dot_1_5 ///
				 consent-dot_1_3_2 dot_1_6 dot_1_7 dot_1_7_oth-dot_1_13_oth dot_1_16 ///
				 key submissiondate starttime endtime today no_consent_why ///
				 no_consent_oth dot_1_9_2 dot_1_9_4-intra_1_26 dot_1_17 ///
				 dot_1_12a-dot_1_12b_oth dot_1_16b

	*** Check and fix duplicates
	duplicates report // all different, 14,325 obs
	duplicates report dot_1_1 // 3,786 duplicates
	duplicates report dot_1_1 call_attempt // still 3,588 duplicates
	duplicates report key
	duplicates report dc_id dot_1_1 resp_city_pulled call_attempt resp_phone_code dot_1_5 ///
					  consent dot_1_3 dot_1_6 dot_1_12 dot_1_16 key dot_1_17 dot_1_10
	duplicates drop   dc_id dot_1_1 resp_city_pulled call_attempt resp_phone_code dot_1_5 ///
					  consent dot_1_3 dot_1_6 dot_1_12 dot_1_16 key dot_1_17 dot_1_10, force
	duplicates tag key if key != "", gen(dup_tag) 
	// duplicated but different conctact info, e.g. phone number, interview time 

	egen 	   nonmiss = rownonmiss(dc_id-dot_1_16b), strok
	egen 	   max = max(nonmiss) if dup_tag == 1, by(dot_1_1) // keeping most complete observations
	drop if    nonmiss != max & dup_tag == 1 // 1,675 obs dropped, 12,650 remaining

	duplicates report key if key != "" // still 1,727 duplicates with diffent phone numbers
	duplicates drop   dc_id dot_1_1 resp_city_pulled call_attempt dot_1_5 consent dot_1_3 ///
					  dot_1_6 dot_1_12 dot_1_16 key dot_1_17 dot_1_10 if key != "", force

	duplicates report key
	duplicates report dot_1_1 key // no duplicate survey entries --> 10,917 obs
	duplicates report dot_1_1 call_attempt // 180 duplicates
	duplicates tag    dc_id dot_1_1 call_attempt, gen(dup_tag2) // 10 duplicates by same enumerator

	sort 	   dot_1_1
	list 	   dc_id dot_1_1 resp_city_pulled call_attempt resp_phone_code dot_1_5 consent ///
			   dot_1_3 dot_1_6 dot_1_12 dot_1_16 dot_1_17 dot_1_10 submissiondate starttime ///
			   endtime if dup_tag2 == 1
	drop if    submissiondate == . & dup_tag2 == 1 & !inlist(_n,3670,3624) // drops the 10 duplicates

	duplicates report dc_id dot_1_1 call_attempt // all unique --> duplicates by different enumerators?
	duplicates tag    dot_1_1 call_attempt, gen(dup_tag3) // still 170 duplicates --> keep most complete
	egen 	   max2 = max(nonmiss) if dup_tag3 == 1, by(dot_1_1)
	drop if    nonmiss != max2 & dup_tag3 == 1 // drops remaining duplicates

	duplicates tag dot_1_1, gen(dup_tag4)
	/* multiple call attempts, with sometimes future calls not accepted because interview already 
	   took place --> again keep most complete observation */
	egen 	   max3 = max(nonmiss) if dup_tag4 == 1, by(dot_1_1)
	drop if    nonmiss != max3 & dup_tag4 == 1
	duplicates report dot_1_1
	drop 	   dup_tag dup_tag2 dup_tag3 dup_tag4 nonmiss max max2 max3
	rename 	   dot_1_1 meta_1_1
	saveold	   "$data_working/Screening_full_nodup.dta", replace version(12.1)
	
	
********************************************************************************
*                          END OF CLEANING DOFILE                              *
********************************************************************************
