********************************************************************************
* Conscious Coupling                                                           *
* Cleaning and Construction: Baseline Datasets                                 *
********************************************************************************


**#  1. Merging Datasets and Cleaning
* -----------------------------------------------------------------------------*	
	use 	     "$data_raw/210927_WB_Couples_Training_BL_Clean_public.dta",    clear 
	append using "$data_raw/220204_WB_Couples_Training_BL_Clean_r2_public.dta", force 
	drop 		 _merge
	
	*** Randomization information
	merge 		 1:1 meta_1_1 key using ///
				 "$data_raw/DOT_Couples_Training_strat_randomization_R2BL.dta", ///
				 keepusing(treatment_new strata_new) keep(1 3) gen(_round2)
	
	* Fix mismatched strata --> value labels not consistent between survey rounds
	label drop 	 strata_new
	replace 	 strata_new = 9  if strata_new == 5
	replace 	 strata_new = 10 if strata_new == 6
	replace 	 strata_new = 11 if strata_new == 7
	replace 	 strata_new = 12 if strata_new == 8
	
	merge 		 1:1 meta_1_1 key using ///
				 "$data_raw/DOT_Couples_Training_strat_randomization.dta", update ///
				 keepusing(treatment_new strata_new) keep(1 4) gen(_round1)
	
	* Final merges
	keep if 	 (_round1 == 4 | _round2 == 3) // removing duplicates
	merge 		 1:1 meta_1_1 using ///
				 "$data_working/Implementation Monitoring_Consolidated_Report.dta", nogen		 
	merge 		 1:1 meta_1_1 using "$data_working/Screening_full_nodup.dta", keep(1 3) nogen
	
	*** Dropping duplicate IDs
	merge 	   	 1:1 meta_1_1 using "$data_raw/230811_WBC_initialduplicatescheck.dta", ///
				 keepusing(new_meta_1_1) keep(1 3) nogen // file shared from data monitoring partner
	merge 	   	 1:1 meta_1_1 using "$data_raw/DOT_duplicates_check.dta", keepusing(Keepdrop) /// 
				 keep(1 3) nogen // created from manual checks, systematically kept first interview

	clonevar   	 meta_1_1_2 = meta_1_1
	replace    	 meta_1_1_2 = new_meta_1_1 if !inlist(new_meta_1_1,0,.)
	replace    	 meta_1_1_2 = 444966       if meta_1_1 == 5361010687 // manual fix
	replace    	 meta_1_1_2 = 1202010030   if inlist(meta_1_1,1202010030,453430)
	replace    	 meta_1_1_2 = 5364020457   if inlist(meta_1_1,5364020457,546115)
	duplicates 	 report meta_1_1
	duplicates 	 report meta_1_1_2 // 14 duplicates

	drop if    	 Keepdrop == "Drop"
	duplicates 	 report meta_1_1_2 // FINAL: 973 actual unique IDs

	rename     	 (meta_1_1 meta_1_1_2) (meta_1_1_old meta_1_1)  
	drop 	   	 Keepdrop new_meta_1_1 meta_1_1_old 
	order 	   	 meta_1_1
	
	*** Checking mistake in values, e.g. typos
	replace 	 b_bus_1_19 = 14 if b_bus_1_19 == -14
	replace 	 meta_1_13  = 2  if inlist(meta_1_1,831710,5374060428)
	// respondents married at endline and answered household questions
	replace 	 meta_1_18  = 1  if meta_1_18 == 0 & meta_1_19 == 1

	keep 		 meta_1_1 strata_new treatment_new Homesession1-any_session today ///
				 meta_1_12a meta_1_10-char_1_53 char_1_66-fin_1_26_3_oth /// 
				 main_business_sector consent dot_1_6 dot_1_12 dot_1_16
				 
	* Save treatment status for easy merging
	preserve
	keep 		 meta_1_1 treatment_new strata_new
	saveold 	 "$data_working/DOT_treatment_status.dta", replace version(12.1)
	restore

	   
**#  2. Creating Main Outcome Variables
* -----------------------------------------------------------------------------*
	gen 		 control = (treatment_new == 0) // control group for computing Anderson indices
	
	* City Indicator for randomization city
	gen 		 city = 1 if inlist(strata_new,1,2,3,4) // Addis Ababa
	replace 	 city = 2 if inlist(strata_new,5,6,7,8) // Bahir Dar
	replace 	 city = 3 if inlist(strata_new,9,10,11,12) // Hawassa
	replace 	 city = 1 if meta_1_12a == 4

	label define city 1 "Addis Ababa" 2 "Bahir Dar" 3 "Hawassa", replace
	label values city city
	  
	
	*** A. Business outcomes and practices
	foreach i in b_bus_1_25 b_bus_1_26 b_bus_1_27 b_bus_1_28 a_bus_1_9_1 a_bus_1_9_2 b_bus_1_31 /// 
				 b_bus_1_33 b_bus_1_35 b_bus_1_37 b_bus_1_50 b_bus_1_41 meta_1_22 meta_1_24 /// 
				 intra_1_33 fin_1_10a fin_1_10b fin_1_10c fin_1_10d fin_1_10d_oth b_bus_1_39 ///
				 main_business_employees main_business_employees_final {
				 label values `i' .
				 recode `i' (-99 -98 -97 -96 -95 -94 .=0) // all numerical values
				 }

	* Adjust monetary values by inflation
	/* Inflation rate (CPI base: December 2016, Source: Ethiopian Statistical Services)
		- Aug 2021: 234.5
		- Sep 2021: 243.8
		- Dec 2021: 245.8
		- Jan 2022: 248.7 */
	gen 		 month = month(today)
	gen 		 year  = year(today)
	foreach i in b_bus_1_25 b_bus_1_26 b_bus_1_27 b_bus_1_28 b_bus_1_31 b_bus_1_33 b_bus_1_35 /// 
				 b_bus_1_37 b_bus_1_41 meta_1_22 meta_1_24 intra_1_33 fin_1_10a fin_1_10b ///
				 fin_1_10c fin_1_10d fin_1_10d_oth {
				 replace `i' = `i'/(2.345) if month == 8  & year == 2021
				 replace `i' = `i'/(2.438) if month == 9  & year == 2021
				 replace `i' = `i'/(2.458) if month == 12 & year == 2021
				 replace `i' = `i'/(2.487) if month == 1  & year == 2022
				 }

	egen 		 machinery = rowtotal(b_bus_1_31 b_bus_1_33)
	clonevar 	 inventory = b_bus_1_35
	clonevar 	 raw = b_bus_1_37
	egen 		 capital = rowtotal(b_bus_1_31 b_bus_1_33 b_bus_1_35 b_bus_1_37)
	
	gen 		 employees = main_business_employees
	replace 	 employees = main_business_employees_final if b_bus_1_15 == 0
	replace 	 employees = 0 if employees == .

	* Winsorinzing and transformations
	foreach i in b_bus_1_25 b_bus_1_26 b_bus_1_27 b_bus_1_28 b_bus_1_39 employees ///
				 capital machinery inventory raw {
				 winsor  `i', gen (win99_`i') p(0.01)
				 winsor  `i', gen (win95_`i') p(0.05)
				 winsor  `i', gen (win90_`i') p(0.10)
				 winsor  `i', gen (win85_`i') p(0.15)
				 winsor  `i', gen (win80_`i') p(0.20)
				 winsor  `i', gen (win75_`i') p(0.25)
				 gen ihs_`i'  = asinh(`i')
				 gen log_`i'  = log(`i')
				 }
	gen 		 any_profits = (b_bus_1_28 > 0)
	gen 		 win95_any_profits = (win95_b_bus_1_28 > 0)

	* Profits quantiles (for heterogeneity)
	label define tertile  0 "Bottom tertile"  1 "Tertile 2"  2 "Top tertile", replace
	label define quintile 0 "Bottom quintile" 1 "Quintile 2" 2 "Quintile 3"  ///
						  3 "Quintile 4" 4 "Top quintile", replace
	
	qui sum 	 win95_b_bus_1_28, d
	gen 		 D_profit = (win95_b_bus_1_28 > r(p50))

	xtile 		 D_profit_3tile = b_bus_1_28, nq(3)
	replace 	 D_profit_3tile = 0 if D_profit_3tile == 1
	replace 	 D_profit_3tile = 1 if D_profit_3tile == 2
	replace 	 D_profit_3tile = 2 if D_profit_3tile == 3
	label values D_profit_3tile tertile

	xtile 		 D_profit_5tile = b_bus_1_28, nq(5)
	replace 	 D_profit_5tile = 0 if D_profit_5tile == 1
	replace 	 D_profit_5tile = 1 if D_profit_5tile == 2
	replace 	 D_profit_5tile = 2 if D_profit_5tile == 3
	replace 	 D_profit_5tile = 3 if D_profit_5tile == 4
	replace 	 D_profit_5tile = 4 if D_profit_5tile == 5
	label values D_profit_5tile quintile

	* Firm size quantiles
	gen 		 D_size = 0 if employees == 0
	replace 	 D_size = 1 if employees > 0 & employees < 6
	replace 	 D_size = 2 if employees >= 6
	label define size 0 "Own-account" 1 "1-5 employees" 2 "6+ employees", replace
	label values D_size size
	gen 		 any_employees = (employees > 0)

	xtile 		 D_size_3tile = employees, nq(3)
	replace 	 D_size_3tile = 0 if D_size_3tile == 1
	replace 	 D_size_3tile = 1 if D_size_3tile == 2
	replace 	 D_size_3tile = 2 if D_size_3tile == 3
	label values D_size_3tile tertile

	* Capital stock quantiles
	qui sum 	 capital, d
	gen 		 D_capital = (capital > r(p50))

	xtile 		 D_capital_3tile = capital, nq(3)
	replace 	 D_capital_3tile = 0 if D_capital_3tile == 1
	replace 	 D_capital_3tile = 1 if D_capital_3tile == 2
	replace 	 D_capital_3tile = 2 if D_capital_3tile == 3
	label values D_capital_3tile tertile

	xtile 	 	 D_capitalstart_3tile = b_bus_1_41, nq(3)
	replace 	 D_capitalstart_3tile = 0 if D_capitalstart_3tile == 1
	replace 	 D_capitalstart_3tile = 1 if D_capitalstart_3tile == 2
	replace 	 D_capitalstart_3tile = 2 if D_capitalstart_3tile == 3
	label values D_capitalstart_3tile tertile
		
	* Business practices
	label define satisfied 1 "Not at all satisfied" 2 "Not very satisfied" ///
						   3 "Fairly satisfifed" 4 "Very satisfied", replace
	clonevar 	 qualbusiness = char_1_68 // business satisfaction
	replace 	 qualbusiness = 5 - qualbusiness if !inlist(char_1_68,0,.)
	label values qualbusiness satisfied

	clonevar 	 D_newbus = b_bus_1_50 // new business
	gen 		 D_aspiration = (b_bus_1_47 == 1) // business aspirations
	gen 		 D_innovation = (b_bus_1_48 == 1 | b_bus_1_49 == 1) // innovation
	
	gen 		 D_busplan = (b_bus_1_22 == 1)
	gen 		 D_budget = (inlist(b_bus_1_23,1,2))
	gen 		 D_records = (b_bus_1_24 != 6)
	egen 		 recordkeep = rowtotal(D_busplan D_budget D_records) // record-keeping

	
	*** B. Socio-emotional skills
	egen 		 selfefficacy = rmean(char_1_2-char_1_11) // self-efficacy
	qui sum 	 selfefficacy, d
	gen 		 D_selfefficacy = (selfefficacy > r(p50))

	egen 		 persgrowth = rmean(char_1_12-char_1_15) // personal growth
	qui sum 	 persgrowth, d
	gen 		 D_persgrowth = (persgrowth > r(p50))

	egen 		 proactivity = rmean(char_1_25-char_1_32) // proactivity
	qui sum 	 proactivity, d
	gen 		 D_proactivity = (proactivity > r(p50))

	egen 		 entidentity = rmean(char_1_44-char_1_45) // entrepreneural identity
	qui sum 	 entidentity, d
	gen 		 D_entidentity = (entidentity > r(p50))

	* Standardized Z-scores
	foreach v in qualbusiness recordkeep selfefficacy persgrowth proactivity entidentity {
				 qui sum `v'
				 gen `v'_z = ((`v' - r(mean))/r(sd))
				 }

	* Composite Anderson indices
	swindex 	 D_busplan D_budget D_records, g(IND_buspractice) normby(control)
	swindex 	 char_1_2-char_1_11 char_1_12-char_1_15 char_1_25-char_1_32, g(IND_ses1) ///
				 normby(control) // removing entrepreneurial identity as only observed at baseline
	clonevar 	 IND_ses_all = IND_ses1
	
	
	*** C. Marriage quality 
	egen 		 coop = rowtotal(intra_1_22_1 intra_1_22_2 intra_1_22_3 intra_1_22_4), missing
	qui sum 	 coop, d // cooperation
	gen 		 D_coop  = (coop > r(p50))
	replace 	 D_coop  = . if meta_1_13 != 2
	gen 		 D_coop0 = (coop  > 0)
	replace 	 D_coop0 = . if meta_1_13 != 2

	egen 		 empathy = rowmean(char_1_46-char_1_50) // empathy
	replace 	 empathy = . if meta_1_13 != 2
	sum 		 empathy, d
	gen 		 D_empathy = (empathy > r(p50))
	replace 	 D_empathy = . if meta_1_13 != 2
	gen 		 D_empathy_low = (empathy >= 3)
	replace 	 D_empathy_low = . if meta_1_13 != 2

	egen 		 commun = rowmean(char_1_52 char_1_53) // communication
	replace 	 commun = . if meta_1_13 != 2
	qui sum 	 commun, d
	gen 		 D_commun = (commun > r(p50))
	replace 	 D_commun = . if meta_1_13 != 2
	gen 		 D_commun_low = (commun >= 3)
	replace 	 D_commun_low = . if meta_1_13 != 2
	
	clonevar 	 intimacy = intra_1_49 // intimate relationship
	replace 	 intimacy = 6 - intimacy if !inlist(intra_1_49,0,.)
	label define intimate 1 "Hostile" 2 "Apathetic" 3 "Loyal" 4 "Congenial" 5 "Intimate", replace
	label values intimacy intimate
	
	clonevar 	 qualmarriage = char_1_16 // marriage satisfaction
	clonevar 	 quallife = char_1_67 // life satisfaction
	replace 	 quallife = 5 - quallife if !inlist(char_1_67,0,.)
	label values quallife satisfied
	
	* Standardized Z-scores
	foreach v in coop empathy commun intimacy qualmarriage quallife {
				 qui sum `v'
				 gen `v'_z = ((`v' - r(mean))/r(sd))
				 }

	* Anderson index
	swindex 	 intra_1_22_1 intra_1_22_2 intra_1_22_3 intra_1_22_4 char_1_46-char_1_50 /// 
				 char_1_52 char_1_53, g(IND_marriage) normby(control)
	replace 	 IND_marriage = . if meta_1_13 != 2
	qui sum 	 IND_marriage, d
	gen 		 D_marriage = (IND_marriage > r(p50))
	replace 	 D_marriage = . if meta_1_13 != 2

	
	*** D. Household outcomes				
	gen 		 inc_share = (meta_1_22/meta_1_24)*100 // income share
	gen 		 D_mainincearn = (meta_1_23 == 1)

	* Winsorizing
	foreach i in meta_1_22 meta_1_24 {
				 winsor  `i', gen (win99_`i') p(0.01)
				 winsor  `i', gen (win95_`i') p(0.05)
				 winsor  `i', gen (win90_`i') p(0.10)
				 winsor  `i', gen (win85_`i') p(0.15)
				 winsor  `i', gen (win80_`i') p(0.20)
				 winsor  `i', gen (win75_`i') p(0.25)
				 gen ihs_`i'  = asinh(`i')
				 gen log_`i'  = log(`i')
				 }
				 
	* Household income quantiles
	xtile 		 D_hhinc_3tile = meta_1_24, nq(3)
	replace 	 D_hhinc_3tile = 0 if D_hhinc_3tile == 1
	replace 	 D_hhinc_3tile = 1 if D_hhinc_3tile == 2
	replace 	 D_hhinc_3tile = 2 if D_hhinc_3tile == 3
	label define income 0 "Low income" 1 "Middle income" 2 "High income", replace
	label values D_hhinc_3tile income


	*** E. Financial outcomes
	clonevar 	 D_loan_all = fin_1_2
	clonevar 	 D_loan = fin_1_11
	clonevar 	 loan_size = fin_1_10a
	gen 		 D_loan_now = (loan_size > 0 & loan_size != .)
	gen 		 D_altfinance = strpos(fin_1_1,"2") > 0 | strpos(fin_1_1,"3") > 0
	egen 		 altfinance_size = rowtotal(fin_1_10b-fin_1_10d_oth)
	replace 	 altfinance_size = 0 if D_altfinance == 0
	
	* Winsorizing
	foreach i in loan_size altfinance_size {
				 winsor  `i', gen (win99_`i') p(0.01)
				 winsor  `i', gen (win95_`i') p(0.05)
				 winsor  `i', gen (win90_`i') p(0.10)
				 winsor  `i', gen (win85_`i') p(0.15)
				 winsor  `i', gen (win80_`i') p(0.20)
				 winsor  `i', gen (win75_`i') p(0.25)
				 gen ihs_`i'  = asinh(`i')
				 gen log_`i'  = log(`i')
				 }

	gen 		 D_WEDP = (fin_1_3 == 1)
	clonevar 	 WEDP_status = fin_1_5
	replace 	 WEDP_status = 0 if WEDP_status == 4
	label define WEDP 0 "No loan or training" 1 "Received loan" 2 "Received training" ///
					  3 "Received loan and training", replace
	label values WEDP_status WEDP
	gen 		 WEDP_loan = 1 if inlist(WEDP_status,1,3)
	replace 	 WEDP_loan = 0 if inlist(WEDP_status,0,2)
	replace 	 WEDP_loan = 0 if D_WEDP == 0


	*** F. Decision-making
	label define decision_making 1 "Self" 2 "Spouse" 3 "Joint HH" 4 "Other HH" ///
								 5 "Joint non-HH" 6 "Other", replace

	foreach i of numlist 4 9 11 12 13 {
		gen 	  decision`i' = . if inlist(intra_1_`i'a,""," ","-97") // missing for non-married women
		replace   decision`i' = 1 if intra_1_`i'a=="1"
		replace   decision`i' = 2 if intra_1_`i'a=="2"
		replace   decision`i' = 3 if ///
		(intra_1_`i'a=="1 2"|intra_1_`i'a=="1 2 3"|intra_1_`i'a=="1 3")
		replace   decision`i' = 4 if (intra_1_`i'a=="3"|intra_1_`i'a=="2 3")
		replace   decision`i' = 5 if ///
		(intra_1_`i'a=="1 4"|intra_1_`i'a=="1 2 4"|intra_1_`i'a=="1 3 4"|intra_1_`i'a=="1 2 3 4")
		replace   decision`i' = 6 if (intra_1_`i'a=="4"|intra_1_`i'a=="2 4"|intra_1_`i'a=="3 4")
		replace   decision`i' = . if meta_1_13 != 2
		label val decision`i' decision_making
	 
		gen 	  D_self_decision`i'  = (inlist(decision`i',1))
		replace   D_self_decision`i'  = . if decision`i' == .
		gen 	  D_husb_decision`i'  = (inlist(decision`i',2))
		replace   D_husb_decision`i'  = . if decision`i' == .
		gen 	  D_joint_decision`i' = (inlist(decision`i',3))
		replace   D_joint_decision`i' = . if decision`i' == .
		}

	gen 		 D_autonomy = (b_bus_1_14 == 1)
	gen 		 D_decisions = (intra_1_17 == 1) // decisions on profits

	* Anderson indices
	swindex 	 D_self_decision9 D_self_decision13, g(IND_self_bus_decision) normby(control)
	swindex 	 D_self_decision4 D_self_decision11 D_self_decision12, ///
				 g(IND_self_hh_decision) normby(control)
	swindex 	 D_husb_decision9 D_husb_decision13, g(IND_husb_bus_decision) normby(control)
	swindex 	 D_husb_decision4 D_husb_decision11 D_husb_decision12, ///
				 g(IND_husb_hh_decision) normby(control)
	swindex 	 D_joint_decision9 D_joint_decision13, g(IND_joint_bus_decision) normby(control)
	swindex 	 D_joint_decision4 D_joint_decision11 D_joint_decision12, ///
				 g(IND_joint_hh_decision) normby(control)
	
	* Simple indices
	egen 		 D_self_bus_decision = anymatch(D_self_decision9 D_self_decision13), v(1)
	replace 	 D_self_bus_decision = . if meta_1_13 != 2
	egen 		 temp = rmean(D_self_decision9 D_self_decision13)
	gen 		 D_self_bus_decision_v2 = (temp == 1)
	replace 	 D_self_bus_decision_v2 = . if meta_1_13 != 2
	drop 		 temp

	egen 		 D_self_hh_decision = anymatch(D_self_decision4 D_self_decision11 ///
				 D_self_decision12), v(1)
	replace 	 D_self_hh_decision = . if meta_1_13 != 2
	egen 		 temp = rmean(D_self_decision4 D_self_decision11 D_self_decision12)
	gen 		 D_self_hh_decision_v2 = (temp == 1)
	replace 	 D_self_hh_decision_v2 = . if meta_1_13 != 2
	drop 		 temp

	egen 		 D_husb_bus_decision = anymatch(D_husb_decision9 D_husb_decision13), v(1)
	replace 	 D_husb_bus_decision = . if meta_1_13 != 2
	egen 		 temp = rmean(D_husb_decision9 D_husb_decision13)
	gen 		 D_husb_bus_decision_v2 = (temp == 1)
	replace 	 D_husb_bus_decision_v2 = . if meta_1_13 != 2
	drop 		 temp

	egen 		 D_husb_hh_decision = anymatch(D_husb_decision4 D_husb_decision11 ///
				 D_husb_decision12), v(1)
	replace 	 D_husb_hh_decision = . if meta_1_13 != 2
	egen 		 temp = rmean(D_husb_decision4 D_husb_decision11 D_husb_decision12)
	gen 		 D_husb_hh_decision_v2 = (temp == 1)
	replace 	 D_husb_hh_decision_v2 = . if meta_1_13 != 2
	drop 		 temp

	egen 		 D_joint_bus_decision = anymatch(D_joint_decision9 D_joint_decision13), v(1)
	replace 	 D_joint_bus_decision = . if meta_1_13 != 2
	egen 		 temp = rmean(D_joint_decision9 D_joint_decision13)
	gen 		 D_joint_bus_decision_v2 = (temp == 1)
	replace 	 D_joint_bus_decision_v2 = . if meta_1_13 != 2
	drop 		 temp

	egen 		 D_joint_hh_decision = anymatch(D_joint_decision4 D_joint_decision11 ///
				 D_joint_decision12), v(1)
	replace 	 D_joint_hh_decision = . if meta_1_13 != 2
	egen 		 temp = rmean(D_joint_decision4 D_joint_decision11 D_joint_decision12)
	gen 		 D_joint_hh_decision_v2 = (temp == 1)
	replace 	 D_joint_hh_decision_v2 = . if meta_1_13 != 2
	drop 		 temp

	
	*** G. Intra-household resource sharing
	foreach i in meta_1_27 meta_1_29 meta_1_28 meta_1_30 intra_1_29 intra_1_32 intra_1_33 {
				 recode `i' (-99 -98 -97 -96 -95 -94 . =0)
				 }

	clonevar 	 time_work_resp = meta_1_27 // time use
	replace  	 time_work_resp = . if meta_1_13 != 2
	clonevar 	 time_work_spouse = meta_1_29
	replace  	 time_work_spouse = . if meta_1_13 != 2
	clonevar 	 time_chores_resp = meta_1_28
	replace  	 time_chores_resp = . if meta_1_13 != 2
	clonevar 	 time_chores_spouse = meta_1_30
	replace  	 time_chores_spouse = . if meta_1_13 != 2

	egen 		 time = rowtotal(time_chores_resp time_chores_spouse), missing
	gen 		 share_chores = time_chores_resp/time
	replace 	 share_chores = 0 if time == 0
	gen 		 share_chores_sp = time_chores_spouse/time
	replace 	 share_chores_sp = 0 if time == 0

	gen 		 D_assetjoint = (intra_1_16 == 3) // asset ownership
	replace 	 D_assetjoint = . if meta_1_13 != 2

	gen 		 D_direct_help = (intra_1_25 == 1 | intra_1_26 == 1) // direct business help
	replace 	 D_direct_help = . if meta_1_13 != 2
	clonevar 	 direct_help = intra_1_29
	replace 	 direct_help = . if meta_1_13 != 2

	gen 		 D_foodprep = (!inlist(intra_1_21a,1,8,9)) // household help
	gen 		 D_clean = (!inlist(intra_1_21b,1,8,9))
	gen 		 D_chcare = (!inlist(intra_1_21c,1,8,9))
	replace 	 D_foodprep = . if meta_1_13 != 2
	replace 	 D_clean = .  if meta_1_13 != 2
	replace 	 D_chcare = . if meta_1_13 != 2 | intra_1_21c == 9

	gen 		 D_foodprep_2 = (inlist(intra_1_21a,1,2)) // this accounts for outside domestic help
	gen 		 D_clean_2 = (inlist(intra_1_21b,1,2))
	gen 		 D_chcare_2 = (inlist(intra_1_21c,1,2))
	replace 	 D_foodprep_2 = . if meta_1_13 != 2
	replace 	 D_clean_2 = .  if meta_1_13 != 2
	replace 	 D_chcare_2 = . if meta_1_13 != 2 | intra_1_21c == 9

	gen 		 D_support_house = (!inlist(intra_1_21a,1,8,9) | !inlist(intra_1_21b,1,8,9) | ///
				 !inlist(intra_1_21c,1,8,9))
	replace 	 D_support_house = . if meta_1_13 != 2
	gen 		 D_support_house_2 = (intra_1_21a == 1 & intra_1_21b == 1 & intra_1_21c == 1)
	replace 	 D_support_house_2 = . if meta_1_13 != 2
	gen 		 D_support_house_3 = (intra_1_21a == 1 | intra_1_21b == 1 | intra_1_21c == 1)
	replace 	 D_support_house_3 = . if meta_1_13 != 2
	gen 		 D_support_house_4 = (inlist(intra_1_21a,1,2) | inlist(intra_1_21b,1,2) | ///
				 inlist(intra_1_21c,1,2))
	replace 	 D_support_house_4 = . if meta_1_13 != 2

	gen 		 D_knowledge = (intra_1_36 == 1) // shared knowledge
	replace 	 D_knowledge = . if meta_1_13 != 2

	gen 		 D_networks = (intra_1_38 == 1) // shared networks
	replace 	 D_networks = . if meta_1_13 != 2

	egen 		 support_emotion = rmean(intra_1_50-intra_1_53) // emotional support
	replace 	 support_emotion = . if meta_1_13 != 2
	qui sum 	 support_emotion
	gen 		 support_emotion_z = ((support_emotion - r(mean))/r(sd))

	gen 		 D_finance = (intra_1_30 == 1 | intra_1_31 == 1) // financial support
	replace 	 D_finance = . if meta_1_13 != 2
	egen 		 support_finance = rowtotal(intra_1_32 intra_1_33)
	replace 	 support_finance = . if meta_1_13 != 2

	* Overall impact (not including help in the household)
	egen 		 D_support_any = anymatch(D_direct_help D_knowledge D_networks D_finance), v(1)
	replace 	 D_support_any = . if meta_1_13 != 2
	
	egen 		 benefit_spouse = rmean(intra_1_46a-intra_1_46f)
	replace 	 benefit_spouse = . if meta_1_13 != 2
	qui sum 	 benefit_spouse
	gen 		 benefit_spouse_z = ((benefit_spouse - r(mean))/r(sd))
	
	* Anderson index
	swindex 	 D_direct_help D_support_house D_knowledge D_networks D_finance ///
				 intra_1_50-intra_1_53, g(IND_support) normby(control)
	replace 	 IND_support = . if meta_1_13 != 2

	* Husbands profile
	label define profile 0 "Indifferent" 1 "Co-preneur" 2 "Regulator" 3 "Opponent", replace
	gen     	 D_husb_profile1 = 0 if D_self_bus_decision == 1 & D_finance == 0
	replace 	 D_husb_profile1 = 1 if D_self_bus_decision == 1 & D_finance == 1
	replace 	 D_husb_profile1 = 2 if D_self_bus_decision == 0 & D_finance == 1
	replace 	 D_husb_profile1 = 3 if D_self_bus_decision == 0 & D_finance == 0
	replace 	 D_husb_profile1 = . if meta_1_13 != 2
	label values D_husb_profile1 profile // wife makes decisions alone (excl. joint)
 
	gen     	 D_husb_profile2 = 0 if D_husb_bus_decision == 0 & D_finance == 0
	replace 	 D_husb_profile2 = 1 if D_husb_bus_decision == 0 & D_finance == 1
	replace 	 D_husb_profile2 = 2 if D_husb_bus_decision == 1 & D_finance == 1
	replace 	 D_husb_profile2 = 3 if D_husb_bus_decision == 1 & D_finance == 0
	replace 	 D_husb_profile2 = . if meta_1_13 != 2
	label values D_husb_profile2 profile // husband does not make decisions alone (incl. joint)


**#  3. Creating Control Variables
* -----------------------------------------------------------------------------*
	gen 		 D_hhead = (meta_1_14 == 1) // household head
	gen 		 D_married = (meta_1_13 == 2) // marital status
	clonevar 	 years_married = intra_1_44
	recode 		 years_married (-99 -98 -97 -96 -95 -94 .=0)
	
	qui sum 	 years_married, d
	gen 		 D_years_married   = (years_married > r(p50))
	gen 		 D_years_married5  = (years_married > 5)
	gen 		 D_years_married10 = (years_married > 10)
	gen 		 D_years_married20 = (years_married > 20)

	xtile 		 D_married_3tile = intra_1_44, nq(3)
	replace 	 D_married_3tile = 0 if D_married_3tile == 1
	replace 	 D_married_3tile = 1 if D_married_3tile == 2
	replace 	 D_married_3tile = 2 if D_married_3tile == 3
	label values D_married_3tile tertile

	clonevar 	 age = meta_1_10
	xtile 		 D_age_3tile = age, nq(3)
	replace 	 D_age_3tile = 0 if D_age_3tile == 1
	replace 	 D_age_3tile = 1 if D_age_3tile == 2
	replace 	 D_age_3tile = 2 if D_age_3tile == 3
	label values D_age_3tile tertile

	clonevar 	 hhsize = meta_1_15 // household size
	clonevar 	 sons = meta_1_16a
	clonevar 	 daughters = meta_1_16b
	clonevar 	 children = children_sum
	recode 		 children (.=0)
	gen 		 D_child5y = (meta_1_17_1 > 0)
	gen 		 D_children = (children > 0)
	sum 		 children, d
	gen 		 D_children2 = (children > r(p50))

	
	*** A. Education
	label define education 0 "No education" 1 "Incomplete primary education" 2 "Primary education" ///
						   3 "High school" 4 "Vocational education" 5 "Other diploma" ///
						   6 "Bachelors degree" 7 "Masters degree" 8 "PhD degree", replace
	clonevar 	 D_literate = meta_1_18
	clonevar 	 D_literate_eng = meta_1_19
	gen 		 educ = .
	replace 	 educ = 0 if meta_1_20 == 1
	replace 	 educ = 1 if inlist(meta_1_20,2,3,4)
	replace 	 educ = 2 if meta_1_20 == 5
	replace 	 educ = 3 if inlist(meta_1_20,6,7,8)
	replace 	 educ = 4 if inlist(meta_1_20,9,10,11,12,13,14,15,16)
	replace 	 educ = 5 if inlist(meta_1_20,17,19)
	replace 	 educ = 6 if inlist(meta_1_20,18,20)
	replace 	 educ = 7 if meta_1_20 == 21
	replace 	 educ = 8 if meta_1_20 == 22
	replace 	 educ = 3 if meta_1_20 == -96 & meta_1_20_oth == "10Th Grade Completed(Old)"
	replace 	 educ = 5 if meta_1_20 == -96 & meta_1_20_oth == "12+2 Diploma"
	label values educ education
	gen 		 D_tertiary = (inlist(educ,4,5,6,7,8))

	gen 		 educ_sp = .
	replace 	 educ_sp = 0 if intra_1_40 == 1
	replace 	 educ_sp = 1 if intra_1_40 == 3
	replace 	 educ_sp = 2 if intra_1_40 == 2
	replace 	 educ_sp = 3 if inlist(intra_1_40,4,5)
	replace 	 educ_sp = 4 if inlist(intra_1_40,9,10,11,12,13,14,15,16)
	replace 	 educ_sp = 5 if inlist(intra_1_40,8,17,19)
	replace 	 educ_sp = 6 if inlist(intra_1_40,6,18)
	replace 	 educ_sp = 7 if intra_1_40 == 7
	replace 	 educ_sp = 3 if intra_1_40 == -96 & intra_1_40_oth == "10Th Completed (Old Curriculum)"
	replace 	 educ_sp = 3 if intra_1_40 == -96 & ///
				 intra_1_40_oth == "10Th Grade Completed (Old Carriculum)"
	replace 	 educ_sp = 3 if intra_1_40 == -96 & intra_1_40_oth == "Preparatory School"
	replace 	 educ_sp = 1 if intra_1_40 == -96 & intra_1_40_oth == "Religes Learn"
	replace 	 educ_sp = 5 if intra_1_40 == -96 & intra_1_40_oth == "12+2 Diploma"
	replace 	 educ_sp = 5 if intra_1_40 == -96 & intra_1_40_oth == "ዲኘሎማ (የሙያ የሆነ )"
	replace 	 educ_sp = 5 if intra_1_40 == -96 & ///
				 inlist(intra_1_40_oth,"10+3","Deploma","Diploma","Diploma(Old)")
	replace 	 educ_sp = 4 if intra_1_40 == -96 & intra_1_40_oth == "Teknikna Muya"
	gen 		 D_tertiary_sp = (inlist(educ_sp,4,5,6,7,8))

	clonevar 	 D_car = intra_1_15_6
	clonevar 	 D_flushtoilet = intra_1_15_7
	clonevar 	 D_computer = intra_1_15_8
	clonevar 	 D_phone = intra_1_15_4
	clonevar 	 D_television = intra_1_15_2
	clonevar 	 D_fridge = intra_1_15_5
	clonevar 	 D_homeown = intra_1_18
	recode 		 D_car D_flushtoilet D_computer D_phone D_television D_fridge (.=0)
	egen 		 hh_assets = rowtotal(intra_1_15_1-intra_1_15_8) // household assets
	
	gen 		 D_training = (meta_1_36 == 1)
	gen 		 D_mindtrain = (inlist(meta_1_37,1,4)) // DOT and TVET College PI
	gen 		 D_mindtrainSU = (dot_1_6 == 1) // DOT and TVET College PI

	
	*** B. Business experience
	gen 		 D_opportunity = (b_bus_1_5 == 1)
	clonevar 	 experience = a_bus_1_1
	qui sum 	 experience, d
	gen 		 D_experience = (experience > r(p50))
	gen 		 D_experience5 = (experience > 5)
	gen 		 age_business = 2013-b_bus_1_3
	qui sum 	 age_business, d
	gen 		 D_age_business = (age_business > r(p50))
	gen 		 D_exp_married = (experience-years_married > 0)
	gen 		 D_bus_married = (age_business-years_married > 0)

	gen 		 D_mentor = (meta_1_33_1 == 1 | meta_1_33_2 == 1)
	gen 		 D_mentor_f = (meta_1_33_2 == 1)
	clonevar 	 mentors = meta_1_34
	recode 		 mentors (-97 =0)
	clonevar 	 clients = b_bus_1_42

	egen 		 digitspan = rowtotal(char_1_17-char_1_24) // digital skills
	qui sum 	 digitspan, d
	gen 		 D_digitspan = (digitspan > r(p50))

	
	*** C. Joint business ownership
	clonevar 	 own1 = b_bus_1_7a_1
	clonevar 	 own2 = b_bus_1_7a_2
	clonevar 	 own3 = b_bus_1_7a_3
	replace 	 own2 = . if b_bus_1_7a_1 == 1 & b_bus_1_7a_2 == 1
	replace 	 own3 = . if b_bus_1_7a_1 == 1 & b_bus_1_7a_3 == 1
	replace 	 own3 = . if b_bus_1_7a_2 == 1 & b_bus_1_7a_3 == 1
	replace 	 own2 = . if b_bus_1_7a_1 == 2 & b_bus_1_7a_2 == 2
	replace 	 own3 = . if b_bus_1_7a_1 == 2 & b_bus_1_7a_3 == 2
	replace 	 own3 = . if b_bus_1_7a_2 == 2 & b_bus_1_7a_3 == 2
	replace 	 b_bus_1_7 = 1 if own2 == . & own3 == .
	list 		 b_bus_1_7 own1 own2 own3 if own2 == . & own3 != . // one case (obs 608)
	replace 	 b_bus_1_7 = 2 if b_bus_1_7 == 1 & own2 != .

	gen 		 D_jointown = ((own1==2|own2==2|own3==2) & (own1==1|own2==1|own3==1))
				 // remove cases where spouse is sole owner
	replace 	 D_jointown = 1 if intra_1_41_oth == "Working Together"
	gen 		 D_owner = (inlist(b_bus_1_8,1,3))

	
	*** D. Spouse employment
	gen 		 D_spousebus = (inlist(intra_1_42,1,2))
	replace 	 D_spousebus = . if meta_1_13 != 2
	gen 		 D_samesector = (intra_1_43 == 1)
	replace 	 D_samesector = . if meta_1_13 != 2

	label define activity_sp 0 "Not working " 1 "Wage employment" 2 "Self-employed", replace
	gen 		 spouse_activity = 0 if inlist(intra_1_41,4,7,8)
	replace 	 spouse_activity = 1 if inlist(intra_1_41,1,2)
	replace 	 spouse_activity = 2 if intra_1_41 == 3 | ///
				 inlist(intra_1_41_oth,"Has Own Church","Ye Insurance Agent","Working Together")
	replace 	 spouse_activity = 0 if intra_1_41 == 6
	replace 	 spouse_activity = 0 if spouse_activity == . & intra_1_41 != . // includes 1 student
	replace 	 spouse_activity = 2 if D_spousebus == 1
	replace 	 spouse_activity = . if inlist(intra_1_41_oth,"He'S Dead","Widowed A Month Before")
	replace 	 spouse_activity = . if meta_1_13 != 2
	label values spouse_activity activity_sp

	gen 		 D_sp_status = 0 if inlist(spouse_activity,0)
	replace 	 D_sp_status = 1 if spouse_activity == 1 & D_sp_status == .
	replace 	 D_sp_status = 3 if (D_samesector == 1 | D_jointown == 1) & D_sp_status == .
	replace 	 D_sp_status = 2 if D_spousebus == 1 & D_sp_status == .
	replace 	 D_sp_status = 0 if D_sp_status == .
	replace 	 D_sp_status = . if meta_1_13 != 2
	label define status 3 "Joint owner/Business in same sector as wife" ///
				 2 "Business owner in other sector" 1 "Salaried worker" 0 "Not working", replace
	label values D_sp_status status

	gen 		 D_sp_status2 = 0 if inlist(spouse_activity,0)
	replace 	 D_sp_status2 = 1 if spouse_activity == 1 & D_sp_status2 == .
	replace 	 D_sp_status2 = 4 if (D_jointown == 1) & D_sp_status2 == .
	replace 	 D_sp_status2 = 3 if (D_samesector == 1) & D_sp_status2 == .
	replace 	 D_sp_status2 = 2 if D_spousebus == 1 & D_sp_status2 == .
	replace 	 D_sp_status2 = 0 if D_sp_status2 == .
	replace 	 D_sp_status2 = . if meta_1_13 != 2
	label define status2 4 "Joint owner" 3 "Business owner in same sector as wife" ///
				 2 "Business owner in different sector as wife" 1 "Salaried worker" ///
				 0 "Not working", replace
	label values D_sp_status2 status2

	gen 		 D_sp_employed = (inlist(spouse_activity,1,2))
	gen 		 D_sp_selfempl = (spouse_activity == 2)
	gen 		 D_sp_salary = (spouse_activity == 1)

	
	*** E. Gender norms/roles
	foreach i in meta_1_31 meta_1_32 char_1_72 char_1_73 b_bus_1_55 b_bus_1_56 {
				 recode `i' (-99 -98 -97 -96 -95 -94 =.)
				 }
	recode 		 meta_1_31 (2 =0)
	
	* Anderson indices
	swindex 	 meta_1_32 char_1_73 b_bus_1_56, g(IND_norms) normby(control) ///
				 flip(meta_1_32 char_1_73 b_bus_1_56) // higher values -> more progressive norms
	swindex 	 meta_1_31 char_1_72 b_bus_1_55, g(IND_beliefs) normby(control) ///
				 flip(meta_1_31 char_1_72 b_bus_1_55)
	swindex 	 D_mainincearn D_hhead D_autonomy, g(IND_roles) normby(control)

	
	*** F. Sectors
	gen 		 sector    = a_bus_1_4_1_1 if a_bus_1_12 == 1
	replace 	 sector    = a_bus_1_4_1_2 if a_bus_1_12 == 2
	replace 	 sector    = a_bus_1_4_1_1 if a_bus_1_12 == .
	gen 		 subsector = a_bus_1_4_2_1 if a_bus_1_12 == 1
	replace 	 subsector = a_bus_1_4_2_2 if a_bus_1_12 == 2
	replace 	 subsector = a_bus_1_4_2_1 if a_bus_1_12 == .

	replace 	 subsector = 32 if sector == 1
	replace 	 subsector = 33 if sector == 3 // separate subsectors for agriculture/construction
	replace 	 subsector = 17 if sector == 6 & subsector == 1
	replace 	 subsector = 1  if sector == 2 & subsector == 17 
	replace 	 sector    = 4  if subsector == 10 & sector == 8
	replace 	 sector    = 4  if subsector == 11 & sector == 8
	replace 	 sector    = 6  if subsector == 16 & sector == 8 
	replace 	 sector    = 6  if subsector == 17 & sector == 8 // fixing sector-subsector links

	* Check specified "other" subsectors
	gen 		 sector_other = a_bus_1_4_2_oth_1 if a_bus_1_12 == 1
	replace 	 sector_other = a_bus_1_4_2_oth_2 if a_bus_1_12 == 2
	replace 	 sector_other = a_bus_1_4_2_oth_1 if a_bus_1_12 == .
	replace 	 subsector    = 17 if inlist(sector_other,"Potato Selling","Food And Cake","Restorant")
	replace 	 subsector    = 1  if sector_other == "Bread"
	replace 	 subsector    = 3  if inlist(sector_other, "Printing", "ህትመት እና እሰቴሸነሪ")
	replace 	 sector       = 2  if inlist(sector_other, "Printing", "ህትመት እና እሰቴሸነሪ")
	replace 	 subsector    = 17 if sector_other == "የምግብ እና መስተንግዶ አቅራቢ"
	replace 	 sector 	  = 6  if sector_other == "የምግብ እና መስተንግዶ አቅራቢ"
	replace 	 subsector 	  = 19 if sector_other == "Film Production"
	replace 	 sector 	  = 7  if sector_other == "Film Production"
	replace 	 subsector 	  = 16 if sector_other == "Room Rent"
	replace 	 subsector 	  = 4  if sector_other == "የፅዳት እቃዎች"
	replace 	 subsector 	  = 34 if sector_other == "የቆዳ ውጤቶች"
	replace 	 subsector 	  = 35 if sector_other == "የእንጨት ስራወች"
	replace 	 subsector 	  = 12 if sector_other == "Bajaj"
	replace 	 subsector 	  = 36 if inlist(sector_other, "Both", "በሁለቱም") & sector == 4
	replace 	 subsector 	  = 11 if sector_other == "የጀምላና ችርቻሮ ሸያጭ"

	label define sector0 1 "Agriculture" 2 "Manufacturing" 3 "Construction" 4 "Wholesale/retail" ///
						 5 "Transport" 6 "Hospitality" 7 "ICT" 8 "Other services", replace
	label copy 	 subsector subsector0
	label define subsector0 32 "Agriculture" 33 "Construction" ///
							27 "Administrative and support services" 31 "Personal services" ///
							19 "TV, sound and music" 15 "Warehousing" 34 "Leather" 35 "Wood work" ///
							36 "Wholesale & retail trade", modify
	label values sector sector0
	label values subsector subsector0
	gen 		 D_wholeretail = (sector == 4)

	* Male-dominated sectors --> using definition from "Crossovers" paper (Alibhai et al. 2017)
	/* - Leather and leather products manufacturing --> 34 Leather
	   - Metal works and engineering --> 6 Metal products
	   - Wood works --> 35 Wood work
	   - Building materials manufacturing --> 5 Rubber & plastics
	   - Gemstone and jewelry production --> -96 Other, 2 Manufacturing (not in sample)
	   - Contracting or subcontracting for construction works --> 33 Construction
	   - Small transport services --> 12 Land, 13 Water, 14 Air transport
	   - Storage and warehousing services --> 15 Warehousing
	   - Tourism services --> 27 Administrative and support services
	   - Staffing/outsourcing services --> 27 Administrative and support services
	   - Engineering services --> ?? 26 Professional, scientific and technical services
	   - Municipal services --> 9 Water supply and waste, 8 Energy production
	   - Automobile maintenance --> 7 Manufacture of transport equipment
	   - Maintenance services (non-automobile) --> ?? 26 Professional, scientific, and technical services
	   - Beekeeping --> 32 Agriculture
	   - Poultry --> ?? 32 Agriculture
	   - Seedlings --> 32 Agriculture
	   - Vegetables and Fruits --> 1 Food, beverages, tobacco */
	gen 		 crossover = (inlist(subsector, 34,6,35,5,33,12,13,14,15,27,26,9,8,7,26,32,1))
	label define cross 0 "Non-crossover" 1 "Crossover", replace
	label values crossover cross
	gen 		 male_dom_sector = (inlist(b_bus_1_13,1,3)) // from survey responses
	replace 	 male_dom_sector = . if b_bus_1_13 == .
	label define male1 0 "Mostly women" 1 "Equal share to mostly men", replace
	label values male_dom_sector male1
	gen 		 male_dom_sector2 = (inlist(b_bus_1_13,1))
	replace 	 male_dom_sector2 = . if b_bus_1_13 == .
	label define male2 0 "Equal share to mostly women" 1 "Mostly men", replace
	label values male_dom_sector2 male2


**#  4. Intermediate Dataset 
* -----------------------------------------------------------------------------*	
	foreach v of varlist _all {
				 label variable `v' "" // survey labels are too long for Stata12 
				 }
	
	saveold		 "$data_working/DOT_Baseline_data.dta", replace version(12.1)
	
	
********************************************************************************
*                        END OF CONSTRUCTION DOFILE                            *
********************************************************************************
