********************************************************************************
* Conscious Coupling                                                           *
* Data Compiling: Final cleaning and labelling variables                       *
********************************************************************************


**#  1. Merging Baseline and Endline Datasets
* -----------------------------------------------------------------------------*
	use 	     "$data_working/DOT_Endline_data.dta", clear
	qui ds 		 meta_1_1 resp_id final_status_main final_status_spouse D_treated D_treated_T1 ///
				 D_treated_T2 flag_treated D_treated_sp treatment_new bus_sector ///
				 bus_register bus_empl resp_age resp_age_pulled strata_new D_ScaleUp-D_ScaleUp_4, not
	foreach v of varlist `r(varlist)' {
				 rename `v' `v'_EL
				 }

	merge 		 1:1 meta_1_1 using "$data_working/DOT_Baseline_data.dta", update keep(3) nogen

	qui ds 		 meta_1_10-main_business_sector control month-male_dom_sector2
	foreach v of varlist `r(varlist)' {
				 rename `v' `v'_BL
				 }


**#  2. Creating Time-Varying Outcomes
* -----------------------------------------------------------------------------*
	gen 		 D_posprofits_EL = (b_bus_1_28_EL > 0 & b_bus_1_28_BL == 0)
	replace 	 D_posprofits_EL = . if final_status_main != "COMPLETED"
	gen 		 D_win95_posprofits_EL = (win95_b_bus_1_28_EL > 0 & win95_b_bus_1_28_BL == 0)
	replace 	 D_win95_posprofits_EL = . if final_status_main != "COMPLETED"

	gen 		 D_hiring_EL = (employees_EL - employees_BL > 0)
	replace 	 D_hiring_EL = 1 if employees_BL == . & employees_EL != .
	replace 	 D_hiring_EL = 0 if employees_BL != . & employees_EL == .
	replace 	 D_hiring_EL = 0 if employees_BL == . & employees_EL == .
	replace 	 D_hiring_EL = . if final_status_main != "COMPLETED"

	gen 		 D_divorce_EL = (meta_1_13_EL == 4 & meta_1_13_BL == 2) // 5% of sample
	replace 	 D_divorce_EL = . if final_status_main != "COMPLETED"
	
	saveold 	 "$data_working/DOT_data_merged.dta", replace version(12.1)


**#  3. Label Variables
* -----------------------------------------------------------------------------*
	keep 		 meta_1_1 final_status_main final_status_spouse treatment_new strata_new ///
				 city Homesession1-any_session dot_1_6-dot_1_16 meta_1_13_* meta_1_22_* ///
				 *_meta_1_22_* meta_1_24_* *_meta_1_24_* meta_1_new7_EL r_businesses_count_* ///
				 b_bus_1_25_* b_bus_1_26_* b_bus_1_27_* b_bus_1_28_* b_bus_1_39_* b_bus_1_41_* ///
				 *_b_bus_1_25_* *_b_bus_1_26_* *_b_bus_1_27_* *_b_bus_1_28_* ///
				 *_b_bus_1_39_* intra_1_new1_1_EL-intra_1_new1_10_EL ///
				 intra_1_new10_EL-intra_1_new13_EL sp_meta_1_20_EL *_sp_meta_1_20_EL ///
				 sp_h_labor_1_16_1_EL sp_h_labor_1_17_1_EL sp_h_labor_1_18_1_EL sp_h_labor_1_20_1_EL ///
				 *_sp_h_labor_1_16_1_EL *_sp_h_labor_1_17_1_EL *_sp_h_labor_1_18_1_EL ///
				 *_sp_h_labor_1_20_1_EL machinery_EL-D_socbias_sp_EL machinery_BL-D_divorce_EL ///
				 meta_1_32_BL char_1_73_BL b_bus_1_56_BL meta_1_31_BL char_1_72_BL b_bus_1_55_BL ///
				 sp_char_1_34_EL sp_meta_1_35_EL sp_intra_1_41_EL sp_intra_1_42_EL sp_intra_1_43_EL ///
				 sp_char_1_33_EL sp_meta_1_34_EL
				  
	drop 		 D_busplan_* D_budget_* D_records_* D_bus_45_1_EL-D_bus_45_13_EL D_bus_45_3_EL ///
				 D_bus_45_9_EL D_bus_45_10_EL D_bus_45_4_EL-D_bus_45_8_EL decision4_* decision9_* ///
				 decision11_* decision12_* decision13_* time_EL time_BL D_car_* D_flushtoilet_* ///
				 D_computer_* D_phone_* D_television_* D_fridge_* own1_* own2_* own3_* date2_EL ///
				 month2_EL sp_decision3_EL sp_decision4_EL sp_decision5_EL sp_decision7_EL time_sp_EL ///
				 sector_other_BL meta_1_24_2_EL meta_1_new7_EL sp_meta_1_24_EL sp_meta_1_22_EL
				 

	*** A. Organizational and screening variables
	label var 	 meta_1_1 			 "Respondent ID"
	label var 	 final_status_main 	 "Main survey data collection status"
	label var 	 final_status_spouse "Husband survey data collection status"
	label var 	 treatment_new 		 "Treatment status"
	label var 	 strata_new 		 "Randomization strata"
	label var 	 city 				 "Randomization city"
	label var 	 Homesession1 		 "Respondent attended home session 1"
	label var 	 Homesession2 		 "Respondent attended home session 2"
	label var 	 Homesession3 		 "Respondent attended home session 3"
	label var 	 SUsession1 		 "Respondent attended ScaleUp session 1"
	label var 	 SUsession2 		 "Respondent attended ScaleUp session 2"
	label var 	 SUsession3 		 "Respondent attended ScaleUp session 3"
	label var 	 n_homesession 		 "Number of home sessions attended"
	label var 	 n_SUsession 		 "Number of ScaleUp sessions attended"
	label var 	 totalsession 		 "Total sessions attended"
	label var 	 complete_HS 		 "Completed all home sessions"
	label var 	 complete_SU 		 "Completed all ScaleUp sessions"
	label var 	 complete_course 	 "Completed all sessions"
	label var 	 any_session 		 "Completed at least one session"
	label var 	 D_ScaleUp 			 "Respondent participated in ScaleUp training"
	label var 	 D_ScaleUp_1 		 "Respondent participated in ScaleUp Health Assessment"
	label var 	 D_ScaleUp_2 		 "Respondent participated in ScaleUp session 1"
	label var 	 D_ScaleUp_3 		 "Respondent participated in ScaleUp session 2"
	label var 	 D_ScaleUp_4 		 "Respondent participated in ScaleUp session 3"
	label var 	 D_treated 			 "Respondent participated in home sessions"
	label var 	 D_treated_T1 		 "Respondent participated in home sessions alone"
	label var 	 D_treated_T2 		 "Respondent participated in home sessions with husband"
	label var 	 flag_treated 		 "Inconsistencies in treatment status recall"
	label var 	 D_treated_sp 		 "Husband participated in at least one home session"
	label var 	 dot_1_6 			 "Previously attended a DOT business training"
	label var 	 dot_1_12 			 "Interested in participating in ScaleUp training"
	label var 	 dot_1_16 			 "Consented to having husband in home sessions"

	
	*** B. Main outcomes
	* Endline only
	label var 	 D_divorce_EL "Respondent is newly divorced or separated"
	label var 	 D_hiring_EL "Any new hirings"
	label var 	 D_posprofits_EL "Shift to positive profits"
	label var 	 D_win95_posprofits_EL "Shift to positive profits (Winsorized 95%)"
	label var 	 D_survival_EL "Respondent's original business is still operating"
	label var 	 D_exit_EL "Respondent has exited from entrepreneurship"
	label var 	 activity_EL "Respondent's primary activity"
	label var 	 D_newsamesector_EL "Respondent opened a new business in the same sector"
	label var 	 D_newdiffsector_EL "Respondent opened a new business in a different sector"
	label var 	 selfesteem_EL "Self-esteem"
	label var 	 cognempathy_EL "Cognitive and emotional empathy"
	label var 	 cognempathy_sjt_EL "Cognitive and emotional empathy, SJT"
	label var 	 express_EL "Expressiveness/assertive communication"
	label var 	 express_sjt_EL "Expressiveness/assertive communication, SJT"
	label var 	 influence_EL "Interpersonal influence, SJT"
	label var 	 negotiate_EL "Negotiation, SJT"
	label var 	 IND_ses2_EL "Interpersonal skills index"
	label var 	 IND_ses_all_v2_EL "Socioemotional skills index (All)"
	label var 	 trust_EL "Trust within marriage"
	label var 	 abuse_EL "Controlling behaviors from husband"
	label var 	 D_abuse_EL "Experiences any controlling behavior from husband"
	label var 	 D_breadwinner_EL "Respondent is the breadwinner"
	label var 	 marketing_EL "Marketing practices"
	label var 	 stockcontrol_EL "Stock control practices"
	label var 	 finplanning_EL "Financial planning practices"
	label var 	 D_interfere_EL "Husband has interfered with her business"
	label var 	 disagree_EL "Number of disagreements between the respondent and her husband"
	label var 	 D_disagree_EL "Respondent has had any disagreement with her husband"
	label var 	 D_hidearn_EL "Respondent wishes to hide earnings from husband"
	label var 	 intra_1_new1_1_EL  "Husband influenced/provided: Gaining new customers/buyers"
	label var 	 intra_1_new1_2_EL  "Husband influenced/provided: Accessing suppliers"
	label var 	 intra_1_new1_3_EL  "Husband influenced/provided: Accessing business loans"
	label var 	 intra_1_new1_4_EL  "Husband influenced/provided: Hiring labor"
	label var 	 intra_1_new1_5_EL  "Husband influenced/provided: Accessing new markets"
	label var 	 intra_1_new1_6_EL  "Husband influenced/provided: Accessing information"
	label var 	 intra_1_new1_7_EL  "Husband influenced/provided: Emotional support"
	label var 	 intra_1_new1_8_EL  "Husband influenced/provided: Cost savings in her business"
	label var 	 intra_1_new1_9_EL  "Husband influenced/provided: Paying rent for her business"
	label var 	 intra_1_new1_10_EL "Husband influenced/provided: Securing location for her business"
	label var 	 IND_support_old_EL "Spousal support index (BL only)"
	label var 	 IND_support_new_EL "Spousal support index (EL only)"
	label var 	 intra_1_new10_EL ///
				 "Women respondent believes would negotiate with their husband for doing chores (Score /10)"
	label var 	 intra_1_new11_EL ///
				 "Women respondent believes would speak badly of a woman whose husband did chores (Score /10)"
	label var 	 intra_1_new12_EL ///
				 "Likelihood of respondent to negotiate with her husband for doing chores"
	label var 	 intra_1_new13_EL ///
				 "Likelihood of respondent to stop working away from home if there were rumors of her using sorcery on her husband"
	label var 	 IND_norms_sjt_EL "Perceived gender norms index, SJT"

	label var 	 savings_EL "Total savings"
	foreach x in 99 95 90 85 80 75 {
				 local varlab: variable label savings_EL
				 label var win`x'_savings_EL "`varlab' (Winsorized `x'%)"
		}
	foreach x in ihs log {
				 local varlab: variable label savings_EL
				 label var `x'_savings_EL "`varlab' (`x')"
		}
		
	foreach x in selfesteem cognempathy cognempathy_sjt express express_sjt influence negotiate ///
				 trust abuse marketing stockcontrol finplanning intra_1_new10 intra_1_new11 /// 
				 intra_1_new12 intra_1_new13 {
				 local varlab: variable label `x'_EL
				 label var `x'_z_EL "`varlab' (Z-score)"
		}
	
	* Baseline only
	label var 	 meta_1_31_BL "Believes that a woman earning more than her husband causes problems"
	label var 	 meta_1_32_BL ///
				 "Neighbours respondent thinks believe a woman earning more than her husband causes problems (Score /10)"
	label var 	 char_1_72_BL "Believes that women should not enter occupations dominated by men"
	label var 	 char_1_73_BL ///
				 "Neighbours respondent thinks believe women should not enter occupations dominated by men (Score /10)"
	label var 	 b_bus_1_55_BL "Believes that household expenses are the responsibility of the man"
	label var 	 b_bus_1_56_BL ///
				 "Neighbours respondent thinks believe household expenses are the responsibility of the man (Score /10)"
	label var 	 IND_beliefs_BL "Personal gender beliefs index"
	label var 	 IND_norms_BL "Perceived gender norms index"
	label var 	 entidentity_BL "Entrepreneurial identity"
	label var 	 entidentity_z_BL "Entrepreneurial identity (Z-score)"
	label var 	 D_selfefficacy_BL "Self efficacy score above median"
	label var 	 D_persgrowth_BL "Personal growth score above median"
	label var 	 D_proactivity_BL "Proactivity score above median"
	label var 	 D_entidentity_BL "Entrepreneurial identity score above median"	
	label var 	 D_WEDP_BL "Respondent is registered with WEDP"
	label var 	 WEDP_status_BL "Respondent's WEDP status"
	label var 	 WEDP_loan_BL "Respondent has a WEDP loan"
	label var 	 D_profit_BL "Annual profits above median"
	label var 	 D_profit_3tile_BL "Annual profits (tertiles)"
	label var 	 D_profit_5tile_BL "Annual profits (quintiles)"
	label var 	 D_size_BL "Firm size"
	label var 	 any_employees_BL "Firm has any employees"
	label var 	 D_size_3tile_BL "Firm size (tertiles)"
	label var 	 D_capital_BL "Capital stock above median"
	label var 	 D_capital_3tile_BL "Capital stock (tertiles)"
	label var 	 D_capitalstart_3tile_BL "Capital stock at startup (tertiles)"
	label var 	 D_hhinc_3tile_BL "Household income (tertiles)"
	label var 	 D_coop_BL "Cooperation score above median"
	label var 	 D_coop0_BL "Has any form of cooperation within marriage"
	label var 	 D_empathy_BL "Empathy score above median"
	label var 	 D_empathy_low_BL "Empathy score above 2 (Almost never)"
	label var 	 D_commun_BL "Communication score above median"
	label var 	 D_commun_low_BL "Empathy score above 2 (Almost never)"

	* Both
	foreach surv in BL EL {
		lab var  D_marriage_`surv' "Marriage quality index above median"
		lab var  IND_roles_`surv' "Actual gender roles index"
		lab var  IND_ses1_`surv' "Intrapersonal skills index"
		lab var  IND_ses_all_`surv' "Socioemotional skills index (All)"
		lab var  any_profits_`surv' "Positive profits"
		lab var  win95_any_profits_`surv' "Positive profits (Winsorized 95%)"
		lab var  b_bus_1_41_`surv' "Start-up capital"
		lab var  D_newbus_`surv' "Opened a new branch or expanded market"
		lab var  D_aspiration_`surv' "Expects business to grow"
		lab var  IND_marriage_`surv' "Marriage quality index"
		lab var  inc_share_`surv' "Percentage of household income contributed by respondent"
		lab var  D_mainincearn_`surv' "Respondent is the main income earner (>80%)"
		lab var  D_loan_all_`surv' "Respondent has ever had a business loan"	
		lab var  D_loan_`surv' "Respondent has applied for a business loan in the last year"
		lab var  D_loan_now_`surv' "Respondent currently has a business loan"
		lab var  D_altfinance_`surv' "Respondent has borrowed money from informal sources"
		lab var  IND_buspractice_`surv' "Business practices index"
		lab var  D_innovation_`surv' "Respondent introduced new products or invested in machinery"
		lab var  D_autonomy_`surv' "Respondent can approve alone major business decisions"
		lab var  D_decisions_`surv' "Respondent decides on use of business profits"
		lab var  time_work_resp_`surv' "Time respondent spends on work for pay"
		lab var  time_work_spouse_`surv' "Time husband spends on work for pay"
		lab var  time_chores_resp_`surv' "Time respondent spends on household chores"
		lab var  time_chores_spouse_`surv' "Time husband spends on household chores"
		lab var  share_chores_`surv' "Wife share of time spent on household chores"
		lab var  share_chores_sp_`surv' "Husband share of time spent on household chores"
		lab var  D_assetjoint_`surv' "Respondent jointly owns household assets with husband"
		lab var  D_direct_help_`surv' "Husband has worked in her business within last year"
		lab var  direct_help_`surv' "Hours per week husband works in her business"
		lab var  D_foodprep_`surv' "Husband helps with preparing food"
		lab var  D_clean_`surv' "Husband helps with cleaning activities"
		lab var  D_chcare_`surv' "Husband helps with taking care of children"
		lab var  D_foodprep_2_`surv' "Respondent does most/all preparing food"
		lab var  D_clean_2_`surv' "Respondent does most/all cleaning activities"
		lab var  D_chcare_2_`surv' "Respondent does most/all taking care of children"
		lab var  D_support_house_`surv' "Husband is involved in any household activity"
		lab var  D_support_house_2_`surv' "Respondent does all household activities"
		lab var  D_support_house_3_`surv' "Respondent does all work in at least one household activity"
		lab var  D_support_house_4_`surv' "Respondent does most to all of any household activity"
		lab var  D_knowledge_`surv' "Husband shared business knowledge"
		lab var  D_networks_`surv' "Husband introduced respondent to their network"
		lab var  D_finance_`surv' "Husband provided financial assistance to her business"
		lab var  support_finance_`surv' "Total value of financial assistance invested in the business"
		lab var  D_support_any_`surv' "Husband provided any support"
		lab var  IND_support_`surv' "Spousal support index"
		
		foreach x in self husb joint {
			if "`x'" == "self"  local tex1 "Respondent"
			if "`x'" == "husb"  local tex2 "Husband"
			if "`x'" == "joint" local tex3 "Household jointly"
			lab var D_`x'_decision9_`surv' "`tex1'`tex2'`tex3' decides on her business activities"
			lab var D_`x'_decision13_`surv' "`tex1'`tex2'`tex3' decides how to use her earnings"
			lab var D_`x'_decision4_`surv' "`tex1'`tex2'`tex3' decides on critical spending matters"
			lab var D_`x'_decision11_`surv' "`tex1'`tex2'`tex3' decides on major household expenses"
			lab var D_`x'_decision12_`surv' "`tex1'`tex2'`tex3' decides on minor household expenses"
			lab var IND_`x'_bus_decision_`surv' "`tex1'`tex2'`tex3' decides on business matters (Index)"
			lab var IND_`x'_hh_decision_`surv' "`tex1'`tex2'`tex3' decides on HH matters (Index)" 
			lab var D_`x'_bus_decision_`surv' "`tex1'`tex2'`tex3' makes decisions on any business matter" 
			lab var D_`x'_hh_decision_`surv' "`tex1'`tex2'`tex3' makes decisions on any HH matter" 
			lab var D_`x'_bus_decision_v2_`surv' "`tex1'`tex2'`tex3' makes decisions on all business matters" 
			lab var D_`x'_hh_decision_v2_`surv' "`tex1'`tex2'`tex3' makes decisions on all HH matters"
			local tex1 ""
			local tex2 ""
			local tex3 ""
			}

		lab var b_bus_1_25_`surv' "Monthly revenues"
		lab var b_bus_1_27_`surv' "Annual revenues"
		lab var b_bus_1_26_`surv' "Monthly profits"
		lab var b_bus_1_28_`surv' "Annual profits"
		lab var b_bus_1_39_`surv' "Monthly expenditures"
		lab var employees_`surv' "Number of employees"
		lab var capital_`surv' "Capital investment"
		lab var machinery_`surv' "Investment in machinery"
		lab var inventory_`surv' "Investment in inventory"
		lab var raw_`surv' "Investment in raw material"
		lab var meta_1_22_`surv' "Respondent's monthly income"
		lab var meta_1_24_`surv' "Household monthly income"
		lab var loan_size_`surv' "Total loans owed"
		lab var altfinance_size_`surv' "Total informal loans owed"	
	
		foreach z in b_bus_1_25 b_bus_1_27 b_bus_1_26 b_bus_1_28 b_bus_1_39 employees capital machinery inventory ///
					 raw meta_1_22 meta_1_24 loan_size altfinance_size {
					 local varlab: variable label `z'_`surv'
					 foreach x in 99 95 90 85 80 75 {
						lab var win`x'_`z'_`surv' "`varlab' (Winsorized `x'%)"
						}
					 foreach x in ihs log {
					 	lab var `x'_`z'_`surv' "`varlab' (`x')"
						}
					}
					
		lab var qualbusiness_`surv' "Satisfaction with business work"
		lab var selfefficacy_`surv' "Self efficacy"
		lab var persgrowth_`surv' "Personal growth"
		lab var proactivity_`surv' "Proactivity"
		lab var coop_`surv' "Cooperation within marriage"
		lab var empathy_`surv' "Empathy in relationship"
		lab var commun_`surv' "Communication with husband"
		lab var intimacy_`surv' "Intimate relationship with husband"
		lab var qualmarriage_`surv' "Satisfaction with married life"
		lab var quallife_`surv' "Satisfaction with economic well-being"
		lab var benefit_spouse_`surv' "Level of benefit to her business from spousal support"
		lab var support_emotion_`surv' "Husband's level of encouragement in her business"
		lab var recordkeep_`surv' "Record-keeping practices"
		foreach x in qualbusiness selfefficacy persgrowth proactivity coop empathy commun ///
					 intimacy qualmarriage quallife benefit_spouse support_emotion recordkeep {
					 local varlab: variable label `x'_`surv'
					 label var `x'_z_`surv' "`varlab' (Z-score)"
					 }
		}


	*** C. Control variables
	* Endline only
	label var 	 socbias_EL "Social desirability bias"
	label var 	 D_socbias_EL "Social desirability bias above median"
	label var 	 socbias_sp_EL "Husband's social desirability bias"
	label var 	 D_socbias_sp_EL "Husband's social desirability bias above median"
	label var 	 HS_complete_sp_EL "Husband completed all home sessions"
	label var 	 D_benefit_sp_EL "Husband feels benefitted from the training"
	label var 	 nontreated_sp_EL "Reason for husband not participating in home session"
	label var 	 D_paychcare_EL "Respondent is willing to pay for childcare"
	label var 	 D_paychcare_sp_EL "Husband is willing to pay for childcare"
	label var	 age_sp_EL "Age of husband"
	label var	 educ_sp_EL "Husband's level of education"
	label var	 D_tertiary_sp_EL "Husband completed tertiary education"
	label var	 experience_sp_EL "Husband's years of entrepreneurial experience"
	label var	 activity_sp_EL "Husband's primary activity"
	label var	 D_opportunity_sp_EL "Husband's business started out of opportunity"
	
	* Baseline only
	label var 	 sector_BL "Sector name"
	label var 	 subsector_BL "Subsector name"
	label var 	 D_wholeretail_BL "Business in wholesale and retail trade sector"
	label var 	 crossover_BL "Operates in male-dominated sector"
	label var 	 male_dom_sector_BL "Perception of male-dominated sector"
	label var	 male_dom_sector2_BL "Perception of male-dominated sector"
	label var 	 D_years_married_BL "Years of marriage above median"
	label var 	 D_years_married5_BL "Married for more than 5 years"
	label var 	 D_years_married10_BL "Married for more than 10 years"
	label var 	 D_years_married20_BL "Married for more than 20 years"
	label var 	 D_married_3tile_BL "Years of marriage cohort (tertiles)"
	label var 	 D_age_3tile_BL "Age cohort (tertiles)"
	label var 	 D_child5y_BL "Any children below 5 years"
	label var 	 D_children2_BL "Number of children above median"
	label var 	 D_literate_BL "Respondent can read and write in at least one language"
	label var 	 D_literate_eng_BL "Respondent can read and write in English"
	label var 	 D_homeown_BL "Respondent is a homeowner"
	label var 	 D_training_BL "Has taken an entrepreneurship training"
	label var 	 D_mindtrain_BL "Has taken a mindset-oriented training"
	label var 	 D_mindtrainSU_BL "Has taken ScaleUp training before"
	label var 	 D_opportunity_BL "Business started out of opportunity"
	label var 	 experience_BL  "Years of entrepreneurial experience"
	label var 	 D_experience_BL "Years of entrepreneurial experience above median"
	label var 	 D_experience5_BL "More than 5 years of entrepreneurial experience"
	label var 	 age_business_BL "Years of current business activity"
	label var 	 D_age_business_BL "Years of business activity above median"
	label var 	 D_exp_married_BL "Was an entrepreneur before marriage"
	label var 	 D_bus_married_BL "Business started before marriage"
	label var 	 D_mentor_BL "Respondent has a business mentor"
	label var 	 D_mentor_f_BL "Respondent has a female business mentor"
	label var 	 digitspan_BL "Number of correct answers on the digit span test"
	label var 	 D_digitspan_BL "Digit span score above median"
	label var 	 D_owner_BL "Respondent is a business owner"
	label var 	 D_sp_employed_BL "Husband is employed"
	label var 	 D_sp_selfempl_BL "Husband is an entrepreneur"
	label var 	 D_sp_salary_BL "Husband is a salaried worker"
	label var 	 D_husb_profile1_BL "Husband support profile (w/o joint decisions)"
	label var 	 D_husb_profile2_BL "Husband support profile (w/ joint decisions)"
	label var	 D_sp_status2_BL "Husband's employment relationship with wife"
	
	
	* Both
	foreach surv in BL EL {
		label var D_married_`surv' "Respondent is married/in a consensual union"
		label var meta_1_13_`surv' "Marital status"
		label var years_married_`surv' "Years of marriage"
		label var D_hhead_`surv' "Respondent is the household head"
		label var age_`surv' "Age of respondent"
		label var hhsize_`surv' "Household size"
		label var D_children_`surv' "Respondent has children"
		label var children_`surv' "Number of children"
		label var sons_`surv' "Number of sons"
		label var daughters_`surv' "Number of daughters"
		label var educ_`surv' "Level of education"
		label var D_tertiary_`surv' "Completed tertiary education"
		label var educ_sp_`surv' "Husband's level of education"
		label var D_tertiary_sp_`surv' "Husband completed tertiary education"
		label var hh_assets_`surv' "Number of household assets owned"
		label var mentors_`surv' "Total number of business owners the respondent consults with"
		label var clients_`surv' "Number of clients per day"
		label var r_businesses_count_`surv' "Number of businesses owned"
		label var D_jointown_`surv' "Business jointly owned with husband"
		label var D_spousebus_`surv' "Husband is a business owner or self-employed"
		label var D_samesector_`surv' "Husband's business is in her same sector"
		label var spouse_activity_`surv' "Husband's primary activity"
		label var D_sp_status_`surv' "Husband's employment relationship with wife"
		}
	
	
	*** D. Husband outcomes
	label var 	 sp_meta_1_34_EL "Husband believes that a woman earning more causes problems"
	label var 	 sp_meta_1_35_EL ///
				 "Neighbours husband thinks believe a woman earning more than her husband causes problems (Score /10)"
	label var 	 sp_char_1_33_EL "Husband believes that men make better business owners than women"
	label var 	 sp_char_1_34_EL ///
				 "Neighbours husband thinks believe men make better business owners than women (Score /10)"
	label var 	 sp_intra_1_41_EL ///
				 "Number of women husband believes would negotiate with their husband for doing chores (Score /10)"
	label var 	 sp_intra_1_42_EL ///
				 "Number of women husband believes would speak badly of a woman whose husband did chores (Score /10)"
	label var 	 sp_intra_1_43_EL "Likelihood of husband to support his wife in doing chores"
	label var 	 IND_beliefs_sp_EL "Husband's personal gender beliefs index"
	label var 	 IND_norms_sp_EL "Husband's perceived gender norms index"
	label var 	 IND_norms_sjt_sp_EL "Husband's perceived gender norms index, SJT"
	label var 	 IND_ses_sp_EL "Husband's socioemotional skills index (All)"
	label var 	 IND_marriage_sp_EL "Husband's marriage quality index"
	label var 	 IND_support_sp_EL "Husband's spousal support index"
	label var 	 time_work_resp_sp_EL "Time respondent spends on work for pay (male perspective)"
	label var 	 time_work_spouse_sp_EL "Time husband spends on work for pay (male perspective)"
	label var 	 time_chores_resp_sp_EL "Time respondent spends on household chores (male perspective)"
	label var	 time_chores_spouse_sp_EL "Time husband spouse spends on household chores (male perspective)"
	label var	 sp_share_chores_EL "Wife share of time spent on household chores (male perspective)"
	label var	 sp_share_chores_sp_EL "Husband share of time spent on household chores (male perspective)"
	label var	 D_assetjoint_sp_EL "Husband jointly owns household assets with wife"
	label var	 D_direct_help_sp_EL "Husband has worked in her business within last year (male perspective)"
	label var	 D_foodprep_sp_EL "Husband helps with preparing food (male perspective)"
	label var	 D_clean_sp_EL "Husband helps with cleaning activities (male perspective)"
	label var	 D_chcare_sp_EL "Husband helps with taking care of children (male perspective)"
	label var	 D_support_house_sp_EL "Husband is involved in any household activity (male perspective)"
	label var	 D_knowledge_sp_EL "Husband shared business knowledge (male perspective)"
	labe var	 D_networks_sp_EL "Husband introduced respondent to their network (male perspective)"
	label var	 support_emotion_sp_EL "Husband's level of encouragement in her business (male perspective)"
	label var	 D_finance_sp_EL "Husband provided financial assistance to her business (male perspective)"
	label var	 D_support_any_sp_EL "Husband provided any support (male perspective)"
	
	foreach x in self husb joint {
		if "`x'" == "self"  local tex1 "Respondent"
		if "`x'" == "husb"  local tex2 "Husband"
		if "`x'" == "joint" local tex3 "Household jointly"
		lab var D_`x'_decision7_sp_EL "`tex1'`tex2'`tex3' decides how to use her earnings (Husband's index)"
		lab var D_`x'_decision3_sp_EL "`tex1'`tex2'`tex3' decides on critical spending matters (Husband's index)"
		lab var D_`x'_decision4_sp_EL "`tex1'`tex2'`tex3' decides on major household expenses (Husband's index)"
		lab var D_`x'_decision5_sp_EL "`tex1'`tex2'`tex3' decides on minor household expenses (Husband's index)"
		lab var IND_`x'_hh_decision_sp_EL "`tex1'`tex2'`tex3' decides on HH matters (Husband's index)" 
		lab var D_`x'_hh_decision_sp_EL "`tex1'`tex2'`tex3' makes decisions on any HH matter (Husband's index)" 
		local tex1 ""
		local tex2 ""
		local tex3 ""
		}

	label var sp_h_labor_1_16_1_EL "Husband's monthly revenues"
	label var sp_h_labor_1_17_1_EL "Husband's monthly profits"
	label var sp_h_labor_1_18_1_EL "Husband's number of employees"
	label var sp_h_labor_1_20_1_EL "Husband's capital investment"
	label var sp_meta_1_20_EL "Husband's monthly earnings"
	foreach z in sp_h_labor_1_16_1 sp_h_labor_1_17_1 sp_h_labor_1_18_1 ///
				 sp_h_labor_1_20_1 sp_meta_1_20 {
				 local varlab: variable label `z'_EL
				 foreach x in 99 95 90 85 80 75 {
					lab var win`x'_`z'_EL "`varlab' (Winsorized `x'%)"
					}
				 foreach x in ihs log {
				 	lab var `x'_`z'_EL "`varlab' (`x')"
					}
				}
					
	label var selfesteem_sp_EL "Husband's self-esteem"
	label var selfefficacy_sp_EL "Husband's self efficacy"
	label var proactivity_sp_EL "Husband's proactivity and personal initiative"
	label var express_sp_EL "Husband's expressiveness/assertive communication"
	label var coop_sp_EL "Husband's cooperation within marriage"
	label var empathy_sp_EL "Husband's empathy in relationship"
	label var commun_sp_EL "Husband's communication with wife"
	label var trust_sp_EL "Husband's trust within marriage"
	label var intimacy_sp_EL "Husband's intimate relationship with wife"
	label var qualmarriage_sp_EL "Husband's satisfaction with married life"
	label var quallife_sp_EL "Husband's satisfaction with economic well-being"
	label var benefit_spouse_sp_EL "Level of benefit to her business from spousal support"
	label var support_emotion_sp_EL "Husband's level of encouragement in her business"
	foreach x in selfesteem_sp selfefficacy_sp proactivity_sp express_sp coop_sp empathy_sp commun_sp ///
				 trust_sp intimacy_sp qualmarriage_sp quallife_sp benefit_spouse_sp support_emotion_sp {
				 local varlab: variable label `x'_EL
				 label var `x'_z_EL "`varlab' (Z-score)"
				 }


**#  4. Final Datasets
* -----------------------------------------------------------------------------*
	order 		 meta_1_1 final_status_main final_status_spouse treatment_new strata_new ///
				 city Homesession1 Homesession2 Homesession3 SUsession1 SUsession2 SUsession3 ///
				 n_homesession n_SUsession totalsession complete_HS complete_SU complete_course ///
				 any_session D_ScaleUp D_ScaleUp_1 D_ScaleUp_2 D_ScaleUp_3 D_ScaleUp_4 D_treated ///
				 D_treated_T1 D_treated_T2 flag_treated dot_1_6 dot_1_12 dot_1_16

	foreach surv in BL EL {
		order 	 meta_1_22_`surv', b(win99_meta_1_22_`surv')
		order 	 meta_1_24_`surv', b(win99_meta_1_24_`surv')
		order 	 b_bus_1_25_`surv', b(win99_b_bus_1_25_`surv')
		order 	 b_bus_1_26_`surv', b(win99_b_bus_1_26_`surv')
		order 	 b_bus_1_27_`surv', b(win99_b_bus_1_27_`surv')
		order 	 b_bus_1_28_`surv', b(win99_b_bus_1_28_`surv')
		order 	 capital_`surv', b(win99_capital_`surv')
		order 	 employees_`surv', b(win99_employees_`surv')
		order 	 b_bus_1_41_`surv', b(capital_`surv')
	}
	order 		 sp_h_labor_1_16_1_EL, b(win99_sp_h_labor_1_16_1_EL)
	order 		 sp_h_labor_1_17_1_EL, b(win99_sp_h_labor_1_17_1_EL)
	order 		 sp_h_labor_1_18_1_EL, b(win99_sp_h_labor_1_18_1_EL)
	order 		 sp_h_labor_1_20_1_EL, b(win99_sp_h_labor_1_20_1_EL)
	order 		 sp_meta_1_20_EL, b(win99_sp_meta_1_20_EL)
	order 		 D_hiring_EL, b(employees_EL)
	order 		 D_divorce_EL, b(D_interfere_EL)
	order 		 D_posprofits_EL D_win95_posprofits_EL, b(b_bus_1_28_EL)

	drop if 	 sector_BL == . // 3 respondents apparently not surveyed at baseline --> everything missing
	notes drop 	 _all
	sort 		 meta_1_1 
	saveold 	 "$data_final/IFWE_DOT_Data.dta", replace version(12.1)
	
	
********************************************************************************
*                        END OF CONSTRUCTION DOFILE                            *
********************************************************************************
