********************************************************************************
* Conscious Coupling                                                           *
* Analysis: Production of descriptive tables in paper and Appendix             *
********************************************************************************
	
	
**#  1. Formatting for Tables
* -----------------------------------------------------------------------------*
	use 	     "$data_final/IFWE_DOT_Data.dta", clear

	
	*** Globals for main outcomes
	#delimit ;
	
		global 	 table1		age_BL hhsize_BL children_BL D_child5y_BL D_tertiary_BL D_mindtrain_BL 
							win95_meta_1_24_BL D_opportunity_BL D_jointown_BL D_wholeretail_BL
							win95_b_bus_1_25_BL win95_b_bus_1_26_BL win95_capital_BL win95_employees_BL
							selfefficacy_z_BL proactivity_z_BL persgrowth_z_BL entidentity_z_BL
							IND_marriage_BL D_spousebus_BL time_work_resp_BL time_chores_resp_BL 
							time_work_spouse_BL time_chores_spouse_BL dot_1_16;

		global 	 table2		win95_b_bus_1_26_BL win95_employees_BL D_wholeretail_BL selfefficacy_z_BL
							IND_marriage_BL D_spousebus_BL time_work_resp_BL time_chores_resp_BL
							D_direct_help_BL D_knowledge_BL D_networks_BL support_emotion_z_BL
							D_finance_BL age_business_BL hhsize_BL D_tertiary_BL;
 
		global 	 tablea2	age_BL hhsize_BL children_BL D_tertiary_BL D_mindtrain_BL 
							win95_meta_1_24_BL D_jointown_BL D_wholeretail_BL
							win95_b_bus_1_25_BL win95_b_bus_1_26_BL win95_capital_BL win95_employees_BL
							selfefficacy_z_BL proactivity_z_BL persgrowth_z_BL entidentity_z_BL
							IND_marriage_BL D_spousebus_BL time_work_resp_BL time_chores_resp_BL 
							time_work_spouse_BL time_chores_spouse_BL;
							
	#delimit cr;

	
	*** Variable changes
	egen 		 takeup = group(any_session treatment_new)
	label define takeup 1 "T1 - Did not start individual training" 2 "T2 - Did not start couples training" ///
						3 "T1 - Attended individual training" 4 "T2 - Attended couples training"
	label values takeup takeup
	
	label define profit	0 "Below median profits" 1 "Above median profits"
	label values D_profit_BL profit
	
	label define treat	0 "Control" 1 "T1 - Scale Up! + individual training" ///
						2 "T2 - Scale Up! + couples training "
	label values treatment_new treat
	
	* Expressing monetary outcomes in Thousands Ethiopian Birr
	foreach v in win95_meta_1_24_BL win95_b_bus_1_25_BL win95_b_bus_1_26_BL ///
				 win95_capital_BL {
				 cap replace `v' = `v'/1000
	}
	
	
	*** Shorter labels for tables
	label var 	 win95_b_bus_1_25_BL "Monthly revenues (in thousand Birr)"
	label var 	 win95_b_bus_1_26_BL "Monthly profits (in thousand Birr)"
	label var 	 win95_employees_BL "Number of employees"
	label var 	 win95_capital_BL "Capital investment (in thousand Birr)"
	label var 	 win95_capital_BL "Household monthly income (in thousand Birr)"
	label var 	 IND_marriage_BL "Marriage quality (Index)"
	label var 	 support_emotion_z_BL "Husband's level of emotional support (Z-score)"
	label var 	 D_direct_help_BL "Husband worked in her business within last year"
	label var 	 time_work_resp_BL "Hours respondent spends on work"
	label var 	 time_chores_resp_BL "Hours respondent spends on chores"
	label var 	 time_work_spouse_BL "Hours spouse spends on work"
	label var 	 time_chores_spouse_BL "Hours spouse spends on chores"  
	
	
	*** Footnotes
	#delimit ;
		
		global 	 foot0		"*** p$<$0.01, ** p$<$0.05, * p$<$0.1";
							
		global 	 foot_bal	"\textit{Notes:} All monetary outcomes are winsorized at the 95\% level. 
							Estimation regressions include city fixed effects. Robust standard errors 
							in parentheses.";
							
		global 	 foot_pr	"\textit{Notes:} Estimation regressions include city fixed effects. 
							Robust standard errors in parentheses.";													
						
	#delimit cr;
		
	
**#  TABLE 1: Randomization Balance
* -----------------------------------------------------------------------------*
	qui reg  	 treatment_new $table1 , robust
	gen 		 reg_samp = 1 if e(sample)

	* Latex
	iebaltab $table1 if reg_samp == 1, grpvar(treatment_new) rowvarlabels vce(robust) ///
		onerow fix(city) stats(pair(nrmd) f(p)) ftest nonote ///
		savetex("$tables_main/Tab1_balance.tex") replace texdoc texlabel("tab:balance") ///
		texcaption("Randomization Balance of Baseline Characteristics") ///
		addnote("$foot_bal" "$foot0")
	
	* Excel
	iebaltab $table1 if reg_samp == 1, grpvar(treatment_new) rowvarlabels vce(robust) ///
		onerow fix(city) stats(pair(nrmd) f(p)) ftest nonote ///
		savexlsx("$tables_main/Tab1_balance.xlsx") replace
		
	drop reg_samp


**#  TABLE 2: Characteristics around Median Profits
* -----------------------------------------------------------------------------*
	qui reg  	 D_profit_BL $table2 , robust
	gen 		 reg_samp = 1 if e(sample)

	* Latex
	iebaltab $table2 if reg_samp == 1, grpvar(D_profit_BL) rowvarlabels vce(robust) ///
		onerow fix(city) stats(pair(nrmd)) nonote ///
		savetex("$tables_main/Tab2_profits.tex") replace texdoc texlabel("tab:bal_profits") ///
		texcaption("Sample Characteristics comparing Firms Above and Below Median Baseline Profits") ///
		addnote("$foot_pr" "$foot0")
	
	* Excel
	iebaltab $table2 if reg_samp == 1, grpvar(D_profit_BL) rowvarlabels vce(robust) ///
		onerow fix(city) stats(pair(nrmd)) nonote ///
		savexlsx("$tables_main/Tab2_profits.xlsx") replace
		
	drop reg_samp


**#  TABLE A1: Take-up Rates
* -----------------------------------------------------------------------------*
	/* Table was constructed manually on Latex using the following items for 
		each cell */
	
	*** Save to matrix
	matrix tableA1 = J(8, 3, .)
	mat colnames tableA1 = treat t1 t2
	
	* Training take-up
	local 		 any_session : variable label any_session
	local 		 complete_HS : variable label complete_HS
	
	* Save totals
	sum 		 treatment_new if inlist(treatment_new,1,2)
	local 		 all `r(N)'
	sum 		 treatment_new if inlist(treatment_new,1,2) & city == 1
	local 		 addis `r(N)'
	sum 		 treatment_new if inlist(treatment_new,1,2) & city == 2
	local 		 bahir `r(N)'
	sum 		 treatment_new if inlist(treatment_new,1,2) & city == 3
	local 		 hawassa `r(N)'
	
	* Populate matrix	
	global treat "inlist(treatment_new,1,2)"
	global t1 "treatment_new == 1"
	global t2 "treatment_new == 2"
	
	foreach col in treat t1 t2 {
		qui sum	 any_session if ${`col'} 
		local 	 m : display %9.0fc (`r(mean)'*100)
		local 	 ind = colnumb(tableA1, "`col'")
		matrix tableA1[1,`ind'] = `m'
		
		qui sum	 complete_HS if ${`col'}
		local 	 m : display %9.0fc (`r(mean)'*100)
		local 	 ind = colnumb(tableA1, "`col'")
		matrix tableA1[2,`ind']  = `m'
		
		qui sum	 any_session if ${`col'} & city == 1
		local 	 m : display %9.0fc (`r(mean)'*100)
		local 	 ind = colnumb(tableA1, "`col'")
		matrix tableA1[3,`ind'] = `m'
		
		qui sum	 complete_HS if ${`col'} & city == 1
		local 	 m : display %9.0fc (`r(mean)'*100)
		local 	 ind = colnumb(tableA1, "`col'")
		matrix tableA1[4,`ind'] = `m'
		
		qui sum	 any_session if ${`col'} & city == 2
		local 	 m : display %9.0fc (`r(mean)'*100)
		local 	 ind = colnumb(tableA1, "`col'")
		matrix tableA1[5,`ind'] = `m'
		
		qui sum	 complete_HS if ${`col'} & city == 2
		local 	 m : display %9.0fc (`r(mean)'*100)
		local 	 ind = colnumb(tableA1, "`col'")
		matrix tableA1[6,`ind'] = `m'
		
		qui sum	 any_session if ${`col'} & city == 3
		local 	 m : display %9.0fc (`r(mean)'*100)
		local 	 ind = colnumb(tableA1, "`col'")
		matrix tableA1[7,`ind'] = `m'
		
		qui sum	 complete_HS if ${`col'} & city == 3
		local 	 m : display %9.0fc (`r(mean)'*100)
		local 	 ind = colnumb(tableA1, "`col'")
		matrix tableA1[8,`ind']	 = `m'
	}
	
	* Extract matrix
	preserve
	
	drop _all
	svmat 		 tableA1, names(col)
	
	gen 		 n = _n
	gen 		 sample = "Respondents invited to training (N = `all')" if inlist(n,1,2)
	replace		 sample = "Addis Ababa (N = `addis')" if inlist(n,3,4)
	replace		 sample = "Bahir Dar (N = `bahir')" if inlist(n,5,6)
	replace		 sample = "Hawassa (N = `hawassa')" if inlist(n,7,8)
	gen 		 training = "`any_session'" if inlist(n,1,3,5,7)
	replace		 training = "`complete_HS'" if inlist(n,2,4,6,8)
	
	* Final compiling
	label var 	 sample "Sample"
	label var 	 training "Training take-up"
	label var 	 treat "Treatment group (%)"
	label var 	 t1 "T1 (%)"
	label var 	 t2 "T2 (%)"
	
	drop 		 n
	order 		 sample training treat t1 t2
	export 		 excel using "$tables_app/TabA1_takeup.xlsx", replace keepcellfmt ///
				 first(varlabels)
	
	restore
	

**#  TABLE A2: Take-up Correlates
* -----------------------------------------------------------------------------*
	preserve
	
	drop if treatment_new == 0 // comparing T1 and T2 groups
	
	qui reg  	 takeup $tablea2 , robust
	gen 		 reg_samp = 1 if e(sample)

	* Latex
	iebaltab $tablea2 if reg_samp == 1, grpvar(takeup) rowvarlabels vce(robust) ///
		onerow fix(city) stats(pair(nrmd) f(p)) ftest nonote ///
		savetex("$tables_app/TabA2_takeup_corr.tex") replace texdoc texlabel("tab:App_takeupcorr") ///
		texcaption("Training Take-up Correlates (T1 vs T2)") ///
		addnote("$foot_bal" "$foot0")
	
	* Excel
	iebaltab $tablea2 if reg_samp == 1, grpvar(takeup) rowvarlabels vce(robust) ///
		onerow fix(city) stats(pair(nrmd)) nonote ///
		savexlsx("$tables_app/TabA2_takeup_corr.xlsx") replace
		
	drop reg_samp
	
	restore


********************************************************************************
*                          END OF ANALYSIS DOFILE                              *
********************************************************************************
