********************************************************************************
* Conscious Coupling                                                           *
* Analysis: Production of main tables in paper 			                       *
********************************************************************************


**#  1. Formatting for Tables
* -----------------------------------------------------------------------------*
	use 	     "$data_final/IFWE_DOT_Data.dta", clear
	
	
	*** Globals for main outcomes
	#delimit ;
		global 	 controls	age_BL years_married_BL experience_BL;

		global 	 business	D_survival win95_b_bus_1_27 win95_b_bus_1_28
							win95_employees;
 
		global 	 invest		win95_capital win95_inventory win95_raw;

		global 	 buspract	IND_buspractice D_innovation D_newbus
							D_newdiffsector;
				 
		global 	 finance	D_loan_now1 D_altfinance win95_loan_size
							win95_savings;
 
		global 	 ses		qualbusiness_z IND_ses_all IND_ses1 IND_ses2;
 
		global 	 decision	D_self_bus_decision D_husb_bus_decision
							D_joint_bus_decision;
 
		global 	 marqual	D_divorce IND_marriage disagree IND_support;
				
		global 	 time		time_work_resp time_chores_resp time_work_spouse
							time_chores_spouse;
	#delimit cr;
	
		
	*** Footnotes
	#delimit ;
	
		global 	 r2 		"Adjusted R$^2$";
		global 	 pval0 		"\textit{p}-value for T1=T2";
		global 	 pval0_1	"p-value for T1=T2";
		global 	 pval1 		"\textit{p}-value for T1 effect";
		global 	 pval1_1	"p-value for T1 effect";
		global 	 pval2 		"\textit{p}-value for T2 effect";
		global 	 pval2_1	"p-value for T2 effect";
		global 	 Est1  		"T1 effect ($ T1+T1*Profits$)";
		global 	 Est1_1		"T1 effect (T1+T1*Profits)";
		global 	 Est2  		"T2 effect ($ T2+T2*Profits$)";
		global 	 Est2_1		"T2 effect (T2+T2*Profits)";
		
		global 	 foot0		"\item \textit{Notes:} *** p$<$0.01, ** p$<$0.05, * p$<$0.1";
		
		global 	 foot1bus	"\item[1] \textit{Business survival} is a dummy for the female entrepreneur's
							main business still being operational at endline; \textit{Annual revenues} 
							indicates the main business' reported revenues in the past year; 
							\textit{Annual profits} indicates the main business' reported profits in the 
							past year; \textit{Number of employees} indicates the total number of current
							employees in the business, not including the female entrepreneur. Continuous
							outcomes are winsorized at the 95\% level.";
 
		global 	 foot1inv	"\item[1] \textit{Total capital investment} indicates the main business' 
							current investment in all owned capital; \textit{Inventory investment} 
							indicates the main business' current investment amount in inventory; 
							\textit{Raw materials investment} indicates the main business' current 
							investment amount in raw materials. Outcomes are winsorized at the 95\% 
							level.";

		global 	 foot1prac	"\item[1] \textit{Business practices} is a composite Anderson index of good
							business practices, including record-keeping, marketing, stock control and 
							financial planning; \textit{Introduced new products} 
							is a dummy equal to 1 if the female entrepreneur developed new products or 
							invested in machinery in the last year, 0 otherwise; 
							\textit{Expanded to new markets} is a dummy equal to 1 if the female 
							entrepreneur opened a new branch or expanded to new markets in the last year,
							0 otherwise; \textit{Shifted to different sector} is a dummy equal to 1 if 
							the female entrepreneur opened a new business in a different sector in the 
							last year, 0 otherwise.";
				 
		global 	 foot1fin	"\item[1] \textit{Currently has a loan} indicates whether the female 
							respondent has a business loan from a formal institution at the time of 
							the survey; \textit{Borrowed informally} indicates whether the respondent has 
							borrowed from informal sources in the last year; \textit{Total loans owed} 
							indicates the total amount of open loans at the 
							time of the survey; \textit{Accumulated savings} indicates the total amount 
							of personal savings at the time of the 
							survey. All continuous outcomes are winsorized at the 95\% level.";
 
		global 	 foot1ses	"\item[1] \textit{Satisfaction with work as business owner} asks female 
							entrepreneurs to rate on a scale from 1 to 5 how satisfied they are with 
							their work as a business owner; \textit{Socioemotional skills} is a composite 
							Anderson index of all intra- (e.g. self-esteem) and inter-personal 
							(e.g. cognitive empathy) skills; \textit{Intrapersonal skills} is a 
							composite Anderson index of intrapersonal skills, including self-esteem, 
							self-efficacy, proactivity and personal growth; 
							\textit{Interpersonal skills} is a composite Anderson index of interpersonal 
							skills, including cognitive empathy, interpersonal influence, assertive 
							communication and negotiation. Twelve observations are missing from the main 
							sample due to a small drop in responses to the socioemotional skills 
							situational judgment tests in the endline survey.";
 
		global 	 foot1dec	"\item[1] \textit{Female entrepreneur sole decisions on her business} is a 
							dummy for female entrepreneurs reporting making decisions by themselves on 
							their business activities and/or use of profits; 
							\textit{Husband sole decisions on her business} is a dummy for female 
							entrepreneurs reporting that their spouse makes decisions on their business 
							activities and/or use of profits; 
							\textit{Joint decisions with husband on her business} is a dummy for female
							entrepreneurs reporting making decisions jointly with their spouse 
							on their business activities and/or use of profits.";
 
		global 	 foot1mar	"\item[1] \textit{Divorced or separated} is a dummy for the female 
							entrepreneur being newly divorced or separated at endline; 
							\textit{Marriage quality} is a composite Anderson index of marital quality,
							including cooperation, empathy in marriage, communication and trust; 
							\textit{Number of disagreements} indicates the recalled number of 
							disagreements with the husbands on her business; 
							\textit{Husband support in business} is a composite Anderson index of spousal 
							support, including financial provision, emotional support, sharing of 
							knowledge and/or network contacts.";
				
		global 	 foot1time	"\item[1] \textit{Time spent by female entrepreneur on work} indicates the
							average hours per day that female entrepreneurs report spending on work for 
							pay activities; \textit{Time spent by female entrepreneur on chores} 
							indicates the average hours per day that female entrepreneurs report spending
							on household activities; \textit{Time spent by husband on work} indicates the 
							average hours per day that female entrepreneurs report that their husband 
							spends on work for pay activities; \textit{Time spent by husband on chores} 
							indicates the average hours per day that female entrepreneurs report their
							husband spends on household activities.";
							
		global 	 foot2qv	"\item[2] Controls include randomization city fixed effects and demographic 
							characteristics (age, years of marriage, entrepreneurial experience). Robust 
							standard errors are reported in round parentheses, while Benjamini Krieger 
							Yekutieli (2006) sharpened q-values are reported in square parentheses 
							(computed globally for all hypotheses and outcomes in the table).";
							
		global 	 foot2noqv	"\item[2] Controls include randomization city fixed effects and demographic 
							characteristics (age, years of marriage, entrepreneurial experience). Robust 
							standard errors are reported in parentheses.";
							
		global 	 foot3_6	"\item[3] For columns (1), (3) and (5), \textit{Control mean} gives the mean
							of the respective outcome at endline for the control group; 
							for columns (2), (4) and (6), it gives the mean of the outcome at endline for 
							the control group for those with below median annual profits.";	
							
		global 	 foot3_8	"\item[3] For columns (1), (3), (5) and (7), \textit{Control mean} gives the 
							mean of the respective outcome at endline for the control group; 
							for columns (2), (4), (6) and (8), it gives the mean of the outcome at 
							endline for the control group for those with below median 
							annual profits.";								
	#delimit cr;
	
	
	*** Variable changes
	rename 		 D_loan_now_EL D_loan_now1_EL
	rename 		 D_loan_all_BL D_loan_now1_BL // D_loan_now_BL is not consistent
	
	gen 		 D_survival_BL      = 0 // no values at baseline
	gen 		 D_newdiffsector_BL = 0 
	gen 		 win95_savings_BL   = 0 
	gen 		 IND_ses2_BL        = 0 
	gen 		 D_divorce_BL       = 0 
	gen 		 disagree_BL        = 0
	
	gen 		 treat1 = (treatment_new == 1) // treatment outcomes
	gen 		 treat2 = (treatment_new == 2)
	gen 		 T1_profit = treat1*D_profit_BL
	gen 		 T2_profit = treat2*D_profit_BL
	
	* Expressing monetary outcomes in Thousands Ethiopian Birr
	foreach surv in BL EL {
		foreach v in win95_b_bus_1_27 win95_b_bus_1_28 win95_capital ///
			  win95_inventory win95_raw win95_loan_size win95_savings {
				 cap replace `v'_`surv' = `v'_`surv'/1000
		}
	}
	
	
	*** Shorter labels for tables
	label var 	 treat1 "T1 - Scale up training"
	label var 	 treat2 "T2 - Scale up + couples training"
	label var 	 T1_profit "T1 * Above median profits"
	label var 	 T2_profit "T2 * Above median profits"
	label var 	 D_survival_EL "Business survival (Yes = 1)"
	label var 	 win95_b_bus_1_27_EL "Annual revenues (Thousand ETB)"
	label var 	 win95_b_bus_1_28_EL "Annual profits (Thousand ETB)"
	label var 	 win95_employees_EL "Number of employees (Count)"
	label var 	 win95_capital_EL "Total capital investment (Thousand ETB)"
	label var 	 win95_inventory_EL "Inventory investment (Thousand ETB)"
	label var 	 win95_raw_EL "Raw materials investment (Thousand ETB)"
	label var 	 IND_buspractice_EL "Business practices (Standardized index)"
	label var 	 D_innovation_EL "Introduced new products (Yes = 1)"
	label var 	 D_newbus_EL "Expanded to new markets (Yes = 1)"
	label var 	 D_newdiffsector_EL "Shifted to different sector (Yes = 1)"
	label var 	 D_loan_now1_EL "Currently has a loan (Yes = 1)"
	label var 	 D_altfinance_EL "Borrowed informally (Yes = 1)"
	label var 	 win95_loan_size_EL "Total loans owed (Thousand ETB)"
	label var  	 win95_savings_EL "Accumulated savings (Thousand ETB)"
	label var 	 qualbusiness_z_EL "Satisfaction with work as business owner (Z-score)"
	label var 	 IND_ses_all_EL "Socioemotional skills (Standardized index)"
	label var 	 IND_ses1_EL "Intrapersonal skills (Standardized index)"
	label var 	 IND_ses2_EL "Interpersonal skills (Standardized index)"
	label var 	 D_self_bus_decision_EL "Female entrepreneur sole decisions on her business (Yes = 1)"
	label var	 D_husb_bus_decision_EL "Husband sole decisions on her business (Yes = 1)"
	label var	 D_joint_bus_decision_EL "Joint decisions with husband on her business (Yes = 1)"
	label var 	 D_divorce_EL "Divorced or separated (Yes = 1)"
	label var 	 IND_marriage_EL "Marriage quality (Standardized index)"
	label var 	 disagree_EL "Number of disagreements (Count)"
	label var 	 IND_support_EL "Husband support in business (Standardized index)"
	label var 	 time_work_resp_EL "Time spent by female entrepreneur on work (Hours per day)"
	label var 	 time_chores_resp_EL "Time spent by female entrepreneur on chores (Hours per day)"
	label var 	 time_work_spouse_EL "Time spent by husband on work (Hours per day)"
	label var 	 time_chores_spouse_EL "Time spent by husband on chores (Hours per day)"


**#  2. Program for Benjamini, Krieger, and Yekutieli (2006) Sharpened Q-Values
* -----------------------------------------------------------------------------*
	/* This code generates sharpened two-stage q-values as described in Anderson (2008).
	   Code is heavily taken from original do-file created by M. Anderson 
	   (last modified by original author: 11/20/07) */
	   
	program drop _all
	
	* Applying sharpened q-values globally on all tested hypotheses
	program define qvals_global, rclass
		// Accept inputs
		syntax , Outcomes(namelist) Treatvars(varlist) Model(string) Fixed(string)

		// Count total tests
		local n_outcomes : word count `outcomes'
		local n_treats : word count `treatvars'
		local total_tests = `n_outcomes' * `n_treats'

		// Initialize matrix to store p-values
		matrix pvals = J(`total_tests', 3, .)
		matrix colnames pvals = outcome treat pval

		// Loop over outcomes
		local row = 1
		foreach y of local outcomes {
			local m_num: list posof `"`y'"' in outcomes
			qui areg `y'_EL `treatvars' `y'_BL $controls `fixed', a(city) robust
			eststo `model'_`m_num'

			foreach t of local treatvars {
				matrix rtable = r(table)
				local t_num : list posof `"`t'"' in treatvars
				local p = rtable["pvalue", "`t'"]
				matrix pvals[`row',1] = `m_num'
				matrix pvals[`row',2] = `t_num'
				matrix pvals[`row',3] = `p'
				local ++row
			}
		}

		// Save data and apply sharpened q-values
		preserve
		drop _all
		svmat pvals, names(col)
		quietly sum pval
		local totalpvals = r(N)

		gen int original_sorting_order = _n
		sort pval
		gen int rank = _n if pval != .
		gen bky06_qval = 1 if pval != .
		local qval = 1

		while `qval' > 0 {
			local qval_adj = `qval' / (1 + `qval')
			gen fdr_temp1 = `qval_adj' * rank / `totalpvals'
			gen reject_temp1 = (fdr_temp1 >= pval) if pval != .
			gen reject_rank1 = reject_temp1 * rank
			egen total_rejected1 = max(reject_rank1)

			local qval_2st = `qval_adj' * (`totalpvals' / (`totalpvals' - total_rejected1[1]))
			gen fdr_temp2 = `qval_2st' * rank / `totalpvals'
			gen reject_temp2 = (fdr_temp2 >= pval) if pval != .
			gen reject_rank2 = reject_temp2 * rank
			egen total_rejected2 = max(reject_rank2)

			replace bky06_qval = `qval' if rank <= total_rejected2 & rank != .
			drop fdr_temp* reject_temp* reject_rank* total_rejected*
			local qval = `qval' - .001
		}
	
		qui sort original_sorting_order
		drop original_sorting_order rank pval
		
		reshape wide bky06_qval, i(outcome) j(treat)
		mkmat bky06_qval*, matrix(sharp_qvals) nomissing
		matrix colnames sharp_qvals = `treatvars'
		matrix rownames sharp_qvals = `outcomes'
		restore

		forvalues i = 1/`n_outcomes' {
			matrix sharp_row = sharp_qvals[`i'..`i',.]
			estadd matrix sharp_qvals = sharp_row : `model'_`i'
		}
	
end

	* Including additional statistics 
	program define addstats, rclass
		// Accept inputs
		syntax , Outcomes(namelist) Treatvars(varlist) Model(string) Fixed(string)

		// Count total outcomes
		local n_outcomes : word count `outcomes'
	
		// Loop over outcomes
		forvalues i = 1/`n_outcomes' {
			local y : word `i' of `outcomes'
			qui areg `y'_EL `treatvars' `y'_BL $controls `fixed', a(city) robust
			
			* 1. Control mean
			if "`model'" == "ate" {
				qui summarize `y'_EL if treat1 == 0 & treat2 == 0
				qui return list 
			}
			if "`model'" == "diff" {
				qui summarize `y'_EL if treat1 == 0 & treat2 == 0 & D_profit_BL == 0
				qui return list 
			}
			estadd scalar control_mean = r(mean): `model'_`i'
			
			* 2. Equality of treatment
			qui lincom treat1 - treat2
			local pval0 = r(p)
			estadd scalar pval0 = `pval0': `model'_`i'
			
			if "`model'" == "diff" {
				
				* 3. Marginal effect of T1 on low profits firms
				qui lincom treat1 + T1_profit
				local Est1 = r(estimate)
				local pval1 = r(p)
				estadd scalar Est1  = `Est1' : `model'_`i'
				estadd scalar pval1 = `pval1': `model'_`i'
				
				* 4. Marginal effect of T2 on low profits firms
				qui lincom treat2 + T2_profit
				local Est2 = r(estimate)
				local pval2 = r(p)
				estadd scalar Est2  = `Est2' : `model'_`i'
				estadd scalar pval2 = `pval2': `model'_`i'
			}
			
			* 5. Model type
			qui summarize `y'_BL
			qui return list
			if `r(mean)' == 0 {
				estadd local ancova = "" : `model'_`i'
			}
			else {
				estadd local ancova = "\checkmark" : `model'_`i'
			}
		}

end

	
**#  TABLE 3. Business Revenues and Profits
* -----------------------------------------------------------------------------*
	eststo clear
	qvals_global, ///
		outcomes($business ) fixed(" ") ///
		treatvars(treat1 treat2) model(ate)
	qvals_global, ///
		outcomes($business ) fixed(" ") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)
	addstats, ///
		outcomes($business ) fixed(" ") ///
		treatvars(treat1 treat2) model(ate)
	addstats, ///
		outcomes($business ) fixed(" ") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)	
		
	local tit "Impact on Business Revenues and Profits"
	local lab "tab:ATE_business"
	
	forvalues i = 1/4 {
		est restore ate_`i' // collect labels for each model
		local depvar = e(depvar)
		local depvar_label : variable label `depvar'
		if "`depvar_label'" == "" local depvar_label "`depvar'"
		local mg_labels_`i' "`depvar_label'"
	}
	
	* Latex
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ///
		using "$tables_main/Tab3_business", replace booktabs ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label mlabels(none) collabels(none) eqlabels(none) ///
		mgroups("`mg_labels_1'" "`mg_labels_2'" "`mg_labels_3'" "`mg_labels_4'", pattern(1 0 1 0 1 0 1 0) ///
		span erepeat(\cmidrule(l){@span}) prefix(\multicolumn{@span}{c}{) suffix(})) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2)) sharp_qvals(par([ ]) fmt(2))) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2 ancova, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f 0) ///
		labels("Observations" "$r2" "Control mean" "$pval0" "$Est1" "$pval1" "$Est2" "$pval2" "ANCOVA model")) nonote ///
		prehead("\begin{table}[!htbp] \begin{adjustbox}{max width=\textwidth} \begin{threeparttable} \caption{`tit'} \label{`lab'} \begin{tabular}{l*{@M}{c}} \toprule \toprule") ///
		postfoot(\bottomrule \end{tabular} \begin{tablenotes} \footnotesize $foot0 $foot1bus $foot2qv $foot3_8 ///
		\end{tablenotes} \end{threeparttable} \end{adjustbox} \end{table})
		
	* Excel
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ///
		using "$tables_main/Tab3_business", replace csv ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label collabels(none) eqlabels(none) wrap ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2)) sharp_qvals(par([ ]) fmt(2))) title(`tit') ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f) ///
		labels("Observations" "Adj. R-squared" "Control mean" "$pval0_1" "$Est1_1" "$pval1_1" "$Est2_1" "$pval2_1")) nonotes
		
		
**#  TABLE 4. Business Investments
* -----------------------------------------------------------------------------*
	eststo clear
	qvals_global, ///
		outcomes($invest ) fixed(" ") ///
		treatvars(treat1 treat2) model(ate)
	qvals_global, ///
		outcomes($invest ) fixed(" ") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)
	addstats, ///
		outcomes($invest ) fixed(" ") ///
		treatvars(treat1 treat2) model(ate)
	addstats, ///
		outcomes($invest ) fixed(" ") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)	
		
	local tit "Impact on Business Investments"
	local lab "tab:ATE_investment"
	
	forvalues i = 1/3 {
		est restore ate_`i' // collect labels for each model
		local depvar = e(depvar)
		local depvar_label : variable label `depvar'
		if "`depvar_label'" == "" local depvar_label "`depvar'"
		local mg_labels_`i' "`depvar_label'"
	}
	
	* Latex
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ///
		using "$tables_main/Tab4_invest", replace booktabs ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label mlabels(none) collabels(none) eqlabels(none) ///
		mgroups("`mg_labels_1'" "`mg_labels_2'" "`mg_labels_3'", pattern(1 0 1 0 1 0) ///
		span erepeat(\cmidrule(l){@span}) prefix(\multicolumn{@span}{c}{) suffix(})) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2)) sharp_qvals(par([ ]) fmt(2))) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2 ancova, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f 0) ///
		labels("Observations" "$r2" "Control mean" "$pval0" "$Est1" "$pval1" "$Est2" "$pval2" "ANCOVA model")) nonote ///
		prehead("\begin{table}[!htbp] \begin{adjustbox}{max width=\textwidth} \begin{threeparttable} \caption{`tit'} \label{`lab'} \begin{tabular}{l*{@M}{c}} \toprule \toprule") ///
		postfoot(\bottomrule \end{tabular} \begin{tablenotes} \footnotesize $foot0 $foot1inv $foot2qv $foot3_6 ///
		\end{tablenotes} \end{threeparttable} \end{adjustbox} \end{table})
		
	* Excel
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ///
		using "$tables_main/Tab4_invest", replace csv ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label collabels(none) eqlabels(none) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2)) sharp_qvals(par([ ]) fmt(2))) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f) ///
		labels("Observations" "Adj. R-squared" "Control mean" "$pval0_1" "$Est1_1" "$pval1_1" "$Est2_1" "$pval2_1")) nonote

		
**#  TABLE 5. Business Practices
* -----------------------------------------------------------------------------*
	eststo clear
	qvals_global, ///
		outcomes($buspract ) fixed(" ") ///
		treatvars(treat1 treat2) model(ate)
	qvals_global, ///
		outcomes($buspract ) fixed(" ") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)
	addstats, ///
		outcomes($buspract ) fixed(" ") ///
		treatvars(treat1 treat2) model(ate)
	addstats, ///
		outcomes($buspract ) fixed(" ") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)	
		
	local tit "Impact on Business Practices"
	local lab "tab:ATE_practices"
	
	forvalues i = 1/4 {
		est restore ate_`i'
		local depvar = e(depvar)
		local depvar_label : variable label `depvar'
		if "`depvar_label'" == "" local depvar_label "`depvar'"
		local mg_labels_`i' "`depvar_label'"
	}
	
	* Latex
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ///
		using "$tables_main/Tab5_buspract", replace booktabs ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label mlabels(none) collabels(none) eqlabels(none) ///
		mgroups("`mg_labels_1'" "`mg_labels_2'" "`mg_labels_3'" "`mg_labels_4'", pattern(1 0 1 0 1 0 1 0) ///
		span erepeat(\cmidrule(l){@span}) prefix(\multicolumn{@span}{c}{) suffix(})) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2)) /*sharp_qvals(par([ ]) fmt(2))*/) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2 ancova, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f 0) ///
		labels("Observations" "$r2" "Control mean" "$pval0" "$Est1" "$pval1" "$Est2" "$pval2" "ANCOVA model")) nonote ///
		prehead("\begin{table}[!htbp] \begin{adjustbox}{max width=\textwidth} \begin{threeparttable} \caption{`tit'} \label{`lab'} \begin{tabular}{l*{@M}{c}} \toprule \toprule") ///
		postfoot(\bottomrule \end{tabular} \begin{tablenotes} \footnotesize $foot0 $foot1prac $foot2noqv $foot3_8 ///
		\end{tablenotes} \end{threeparttable} \end{adjustbox} \end{table})
		
	* Excel
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ///
		using "$tables_main/Tab5_buspract", replace csv ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label collabels(none) eqlabels(none) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2)) /*sharp_qvals(par([ ]) fmt(2))*/) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f) ///
		labels("Observations" "Adj. R-squared" "Control mean" "$pval0_1" "$Est1_1" "$pval1_1" "$Est2_1" "$pval2_1")) nonote
		 
		 
**#  TABLE 6. Financial Outcomes
* -----------------------------------------------------------------------------*
	eststo clear
	qvals_global, ///
		outcomes($finance ) fixed(" ") ///
		treatvars(treat1 treat2) model(ate)
	qvals_global, ///
		outcomes($finance ) fixed(" ") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)
	addstats, ///
		outcomes($finance ) fixed(" ") ///
		treatvars(treat1 treat2) model(ate)
	addstats, ///
		outcomes($finance ) fixed(" ") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)	
		
	local tit "Impact on Financial Outcomes"
	local lab "tab:ATE_finance"
	
	forvalues i = 1/4 {
		est restore ate_`i'
		local depvar = e(depvar)
		local depvar_label : variable label `depvar'
		if "`depvar_label'" == "" local depvar_label "`depvar'"
		local mg_labels_`i' "`depvar_label'"
	}
	
	* Latex
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ///
		using "$tables_main/Tab6_finance", replace booktabs ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label mlabels(none) collabels(none) eqlabels(none) ///
		mgroups("`mg_labels_1'" "`mg_labels_2'" "`mg_labels_3'" "`mg_labels_4'", pattern(1 0 1 0 1 0 1 0) ///
		span erepeat(\cmidrule(l){@span}) prefix(\multicolumn{@span}{c}{) suffix(})) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2)) /*sharp_qvals(par([ ]) fmt(2))*/) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2 ancova, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f 0) ///
		labels("Observations" "$r2" "Control mean" "$pval0" "$Est1" "$pval1" "$Est2" "$pval2" "ANCOVA model")) nonote ///
		prehead("\begin{table}[!htbp] \begin{adjustbox}{max width=\textwidth} \begin{threeparttable} \caption{`tit'} \label{`lab'} \begin{tabular}{l*{@M}{c}} \toprule \toprule") ///
		postfoot(\bottomrule \end{tabular} \begin{tablenotes} \footnotesize $foot0 $foot1fin $foot2noqv $foot3_8 ///
		\end{tablenotes} \end{threeparttable} \end{adjustbox} \end{table})
		
	* Excel
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ///
		using "$tables_main/Tab6_finance", replace csv ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label collabels(none) eqlabels(none) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2)) /*sharp_qvals(par([ ]) fmt(2))*/) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f) ///
		labels("Observations" "Adj. R-squared" "Control mean" "$pval0_1" "$Est1_1" "$pval1_1" "$Est2_1" "$pval2_1")) nonote
		
		
**#  TABLE 7. Socioemotional Skills
* -----------------------------------------------------------------------------*
	eststo clear
	
	* Make balanced sample
	qui reg treatment_new selfesteem_EL selfefficacy_EL persgrowth_EL proactivity_EL ///
				cognempathy_sjt_EL express_sjt_EL influence_EL negotiate_EL, robust
	gen reg_samp = 1 if e(sample)
	
	qvals_global, ///
		outcomes($ses ) ///
		treatvars(treat1 treat2) model(ate) fixed("if reg_samp == 1")
	qvals_global, ///
		outcomes($ses ) fixed("if reg_samp == 1") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)
	addstats, ///
		outcomes($ses ) ///
		treatvars(treat1 treat2) model(ate) fixed("if reg_samp == 1")
	addstats, ///
		outcomes($ses ) fixed("if reg_samp == 1") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)	
		
	local tit "Impact on Socioemotional Skills"
	local lab "tab:ATE_ses"
	
	forvalues i = 1/4 {
		est restore ate_`i'
		local depvar = e(depvar)
		local depvar_label : variable label `depvar'
		if "`depvar_label'" == "" local depvar_label "`depvar'"
		local mg_labels_`i' "`depvar_label'"
	}
	
	* Latex
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ///
		using "$tables_main/Tab7_ses", replace booktabs ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label mlabels(none) collabels(none) eqlabels(none) ///
		mgroups("`mg_labels_1'" "`mg_labels_2'" "`mg_labels_3'" "`mg_labels_4'", pattern(1 0 1 0 1 0 1 0) ///
		span erepeat(\cmidrule(l){@span}) prefix(\multicolumn{@span}{c}{) suffix(})) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2)) /*sharp_qvals(par([ ]) fmt(2))*/) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2 ancova, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f 0) ///
		labels("Observations" "$r2" "Control mean" "$pval0" "$Est1" "$pval1" "$Est2" "$pval2" "ANCOVA model")) nonote ///
		prehead("\begin{table}[!htbp] \begin{adjustbox}{max width=\textwidth} \begin{threeparttable} \caption{`tit'} \label{`lab'} \begin{tabular}{l*{@M}{c}} \toprule \toprule") ///
		postfoot(\bottomrule \end{tabular} \begin{tablenotes} \footnotesize $foot0 $foot1ses $foot2noqv $foot3_8 ///
		\end{tablenotes} \end{threeparttable} \end{adjustbox} \end{table})
		
	* Excel
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ///
		using "$tables_main/Tab7_ses", replace csv ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label collabels(none) eqlabels(none) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2)) /*sharp_qvals(par([ ]) fmt(2))*/) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f) ///
		labels("Observations" "Adj. R-squared" "Control mean" "$pval0_1" "$Est1_1" "$pval1_1" "$Est2_1" "$pval2_1")) nonote
		
		
**#  TABLE 8. Decision-Making
* -----------------------------------------------------------------------------*
	drop reg_samp
	eststo clear
	qvals_global, ///
		outcomes($decision ) fixed(" ") ///
		treatvars(treat1 treat2) model(ate)
	qvals_global, ///
		outcomes($decision ) fixed(" ") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)
	addstats, ///
		outcomes($decision ) fixed(" ") ///
		treatvars(treat1 treat2) model(ate)
	addstats, ///
		outcomes($decision ) fixed(" ") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)	
		
	local tit "Impact on Decision-Making Power"
	local lab "tab:ATE_decision"
	
	forvalues i = 1/3 {
		est restore ate_`i'
		local depvar = e(depvar)
		local depvar_label : variable label `depvar'
		if "`depvar_label'" == "" local depvar_label "`depvar'"
		local mg_labels_`i' "`depvar_label'"
	}
	
	* Latex
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ///
		using "$tables_main/Tab8_decisions", replace booktabs ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label mlabels(none) collabels(none) eqlabels(none) ///
		mgroups("`mg_labels_1'" "`mg_labels_2'" "`mg_labels_3'" "`mg_labels_4'", pattern(1 0 1 0 1 0 1 0) ///
		span erepeat(\cmidrule(l){@span}) prefix(\multicolumn{@span}{c}{) suffix(})) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2)) /*sharp_qvals(par([ ]) fmt(2))*/) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2 ancova, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f 0) ///
		labels("Observations" "$r2" "Control mean" "$pval0" "$Est1" "$pval1" "$Est2" "$pval2" "ANCOVA model")) nonote ///
		prehead("\begin{table}[!htbp] \begin{adjustbox}{max width=\textwidth} \begin{threeparttable} \caption{`tit'} \label{`lab'} \begin{tabular}{l*{@M}{c}} \toprule \toprule") ///
		postfoot(\bottomrule \end{tabular} \begin{tablenotes} \footnotesize $foot0 $foot1dec $foot2noqv $foot3_6 ///
		\end{tablenotes} \end{threeparttable} \end{adjustbox} \end{table})
		
	* Excel
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ///
		using "$tables_main/Tab8_decisions", replace csv ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label collabels(none) eqlabels(none) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2)) /*sharp_qvals(par([ ]) fmt(2))*/) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f) ///
		labels("Observations" "Adj. R-squared" "Control mean" "$pval0_1" "$Est1_1" "$pval1_1" "$Est2_1" "$pval2_1")) nonote
		
		
**#  TABLE 9. Marriage Quality
* -----------------------------------------------------------------------------*
	eststo clear
	qvals_global, ///
		outcomes($marqual ) fixed(" ") ///
		treatvars(treat1 treat2) model(ate)
	qvals_global, ///
		outcomes($marqual ) fixed(" ") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)
	addstats, ///
		outcomes($marqual ) fixed(" ") ///
		treatvars(treat1 treat2) model(ate)
	addstats, ///
		outcomes($marqual ) fixed(" ") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)	
		
	local tit "Impact on Marriage Quality Outcomes"
	local lab "tab:ATE_marriage"
	
	forvalues i = 1/4 {
		est restore ate_`i'
		local depvar = e(depvar)
		local depvar_label : variable label `depvar'
		if "`depvar_label'" == "" local depvar_label "`depvar'"
		local mg_labels_`i' "`depvar_label'"
	}
	
	* Latex
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ///
		using "$tables_main/Tab9_marriage", replace booktabs ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label mlabels(none) collabels(none) eqlabels(none) ///
		mgroups("`mg_labels_1'" "`mg_labels_2'" "`mg_labels_3'" "`mg_labels_4'", pattern(1 0 1 0 1 0 1 0) ///
		span erepeat(\cmidrule(l){@span}) prefix(\multicolumn{@span}{c}{) suffix(})) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2)) /*sharp_qvals(par([ ]) fmt(2))*/) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2 ancova, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f 0) ///
		labels("Observations" "$r2" "Control mean" "$pval0" "$Est1" "$pval1" "$Est2" "$pval2" "ANCOVA model")) nonote ///
		prehead("\begin{table}[!htbp] \begin{adjustbox}{max width=\textwidth} \begin{threeparttable} \caption{`tit'} \label{`lab'} \begin{tabular}{l*{@M}{c}} \toprule \toprule") ///
		postfoot(\bottomrule \end{tabular} \begin{tablenotes} \footnotesize $foot0 $foot1mar $foot2noqv $foot3_8 ///
		\end{tablenotes} \end{threeparttable} \end{adjustbox} \end{table})
		
	* Excel
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ///
		using "$tables_main/Tab9_marriage", replace csv ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label collabels(none) eqlabels(none) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2)) /*sharp_qvals(par([ ]) fmt(2))*/) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f) ///
		labels("Observations" "Adj. R-squared" "Control mean" "$pval0_1" "$Est1_1" "$pval1_1" "$Est2_1" "$pval2_1")) nonote
		
		
**#  TABLE 10. Time Use
* -----------------------------------------------------------------------------*
	eststo clear
	qvals_global, ///
		outcomes($time ) fixed(" ") ///
		treatvars(treat1 treat2) model(ate)
	qvals_global, ///
		outcomes($time ) fixed(" ") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)
	addstats, ///
		outcomes($time ) fixed(" ") ///
		treatvars(treat1 treat2) model(ate)
	addstats, ///
		outcomes($time ) fixed(" ") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)	
		
	local tit "Impact on Time Use within the Household"
	local lab "tab:ATE_time"
	
	forvalues i = 1/4 {
		est restore ate_`i'
		local depvar = e(depvar)
		local depvar_label : variable label `depvar'
		if "`depvar_label'" == "" local depvar_label "`depvar'"
		local mg_labels_`i' "`depvar_label'"
	}
	
	* Latex
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ///
		using "$tables_main/Tab10_time", replace booktabs ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label mlabels(none) collabels(none) eqlabels(none) ///
		mgroups("`mg_labels_1'" "`mg_labels_2'" "`mg_labels_3'" "`mg_labels_4'", pattern(1 0 1 0 1 0 1 0) ///
		span erepeat(\cmidrule(l){@span}) prefix(\multicolumn{@span}{c}{) suffix(})) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2)) /*sharp_qvals(par([ ]) fmt(2))*/) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2 ancova, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f 0) ///
		labels("Observations" "$r2" "Control mean" "$pval0" "$Est1" "$pval1" "$Est2" "$pval2" "ANCOVA model")) nonote ///
		prehead("\begin{table}[!htbp] \begin{adjustbox}{max width=\textwidth} \begin{threeparttable} \caption{`tit'} \label{`lab'} \begin{tabular}{l*{@M}{c}} \toprule \toprule") ///
		postfoot(\bottomrule \end{tabular} \begin{tablenotes} \footnotesize $foot0 $foot1time $foot2noqv $foot3_8 ///
		\end{tablenotes} \end{threeparttable} \end{adjustbox} \end{table})
		
	* Excel
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ///
		using "$tables_main/Tab10_time", replace csv ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label collabels(none) eqlabels(none) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2)) /*sharp_qvals(par([ ]) fmt(2))*/) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f) ///
		labels("Observations" "Adj. R-squared" "Control mean" "$pval0_1" "$Est1_1" "$pval1_1" "$Est2_1" "$pval2_1")) nonote
		

********************************************************************************
*                          END OF ANALYSIS DOFILE                              *
********************************************************************************
