********************************************************************************
* Conscious Coupling                                                           *
* Analysis: Production of tables in Appendix       				               *
********************************************************************************


**#  1. Formatting for Tables
* -----------------------------------------------------------------------------*
	use 	     "$data_final/IFWE_DOT_Data.dta", clear
	

	*** Globals for main outcomes
	#delimit ;
		global 	 controls	i.city age_BL years_married_BL experience_BL;

		global 	 attrition	attrition_main attrition_spouse;
		
		global 	 spouse		win95_sp_h_labor_1_16_1 win95_sp_h_labor_1_17_1 
							win95_sp_h_labor_1_20_1 IND_marriage_sp;
							
		global 	 business2	win95_b_bus_1_25 win95_b_bus_1_26 win95_b_bus_1_39;
 
		global 	 ses2		cognempathy_sjt_z express_sjt_z influence_z negotiate_z;

		global 	 norms		IND_norms_sjt intra_1_new10_z intra_1_new11_z intra_1_new12_z 
							intra_1_new13_z;
							
		global 	 business	D_survival win95_b_bus_1_27 win95_b_bus_1_28
							win95_employees;
				 
		global 	 finance	D_loan_now1 D_altfinance win95_loan_size
							win95_savings;
	#delimit cr;	

	
	*** Footnotes
	#delimit ;
	
		global 	 r2 		"Adjusted R$^2$";
		global 	 pval0 		"\textit{p}-value for T1=T2";
		global 	 pval0_1	"p-value for T1=T2";
		global 	 pval1 		"\textit{p}-value for T1 effect";
		global 	 pval1_1	"p-value for T1 effect";
		global 	 pval2 		"\textit{p}-value for T2 effect";
		global 	 pval2_1	"p-value for T2 effect";
		global 	 Est1  		"T1 effect ($ T1+T1*Profits$)";
		global 	 Est1_1		"T1 effect (T1+T1*Profits)";
		global 	 Est2  		"T2 effect ($ T2+T2*Profits$)";
		global 	 Est2_1		"T2 effect (T2+T2*Profits)";
		
		global 	 foot0		"\item \textit{Notes:} *** p$<$0.01, ** p$<$0.05, * p$<$0.1";
		
		global 	 foot1att	"\item[1] Covariates are all in relation to the female entrepreneur. 
							Robust standard errors in parentheses.";
							
		global 	 foot2att	"\item[2] For columns (1) and (3), \textit{Control mean} gives the mean 
							attrition rate for the control group; for columns (2) and (4), it gives 
							the mean attrition for the control group for those with below median 
							annual profits.";
							
		global 	 foot1sp	"\item[1] \textit{Husband's monthly revenues} indicates the husband business' 
							reported revenues in the past month; \textit{Husband's monthly profits} indicates 
							the husband business' reported profits in the past month; 
							\textit{Husband's capital investment} indicates the husband business' current 
							investment in all owned capital; \textit{Husband's marriage quality} is a 
							composite Anderson index of marital quality, including cooperation, empathy in 
							marriage, communication and trust from the husband's perspective. All outcomes 
							are observed only at endline and monetary outcomes are winsorized at the 
							95\% level.";
							
		global 	 foot1bus2	"\item[1] \textit{Monthly revenues} indicates the main business' reported revenues 
							in the past month; \textit{Monthly profits} indicates the main business' reported 
							profits in the past month; \textit{Monthly costs} indicates the main business' 
							reported total expenditures in the past month. Outcomes are winsorized at the 
							95\% level.";
							
		global 	 foot1ses	"\item[1] \textit{Cognitive empathy} indicates the female respondent's ability 
							to understand another's viewpoint and have emotional concern for another's 
							situation or experience; \textit{Expressiveness} indicates the female respondent's 
							ability to explain ideas and openly expressing her opinions in an assertive manner; 
							\textit{Interpersonal influence} indicates the respondent's ability to communicate 
							in a manner that changes other's perspectives, particularly within marital 
							relationships; \textit{Negotiation} indicates the respondent's ability to identify 
							her own and other's interests during a disagreement and to change her own behaviors, 
							thoughts and feelings as a strategy for resolving interpersonal problems. 
							All outcomes are measured using situational judgment tests at endline. Twelve 
							observations are missing from the main sample due to a small drop in responses 
							to the socioemotional skills situational judgment tests in the endline survey.";
							
		global 	 foot1norm	"\item[1] \textit{Perceived gender norms} is a composite Anderson index capturing
							the female respondent's perception of social norms in her community around women
							in business, where higher values indicate more progressive social norms. Each 
							component is measured using situational judgment tests at endline, where respondents
							are asked to imagine a woman named Selam, who is married with two young children, 
							and has a retail business with an opportunity to increase revenues, but it would 
							involved early morning work. Respondents are asked to think of the reactions of 
							ten women in her community, whether positive or negative.";
		
		global 	 foot1bus	"\item[1] \textit{Business survival} is a dummy for the female entrepreneur's
							main business still being operational at endline; \textit{Annual revenues} 
							indicates the main business' reported revenues in the past year; 
							\textit{Annual profits} indicates the main business' reported profits in the 
							past year; \textit{Number of employees} indicates the total number of current
							employees in the business, not including the female entrepreneur. Continuous
							outcomes are winsorized at the 95\% level.";
				 
		global 	 foot1fin	"\item[1] \textit{Currently has a loan} indicates whether the female 
							respondent has a business loan from a formal institution at the time of 
							the survey; \textit{Borrowed informally} indicates whether the respondent has 
							borrowed from informal sources in the last year; \textit{Total loans owed} 
							indicates the total amount of open loans at the 
							time of the survey; \textit{Accumulated savings} indicates the total amount 
							of personal savings at the time of the 
							survey. All continuous outcomes are winsorized at the 95\% level.";
							
		global 	 foot2		"\item[2] Controls include randomization city fixed effects and demographic 
							characteristics (age, years of marriage, entrepreneurial experience). Robust 
							standard errors are reported in parentheses.";
							
		global 	 foot3_6	"\item[3] For columns (1), (3) and (5), \textit{Control mean} gives the mean
							of the respective outcome at endline for the control group; 
							for columns (2), (4) and (6), it gives the mean of the outcome at endline for 
							the control group for those with below median annual profits.";	
							
		global 	 foot3_8	"\item[3] For columns (1), (3), (5) and (7), \textit{Control mean} gives the 
							mean of the respective outcome at endline for the control group; 
							for columns (2), (4), (6) and (8), it gives the mean of the outcome at 
							endline for the control group for those with below median 
							annual profits.";
							
		global 	 foot3_10	"\item[3] For columns (1), (3), (5), (7) and (9), \textit{Control mean} gives the 
							mean of the respective outcome at endline for the control group; 
							for columns (2), (4), (6), (8) and (10), it gives the mean of the outcome at 
							endline for the control group for those with below median 
							annual profits.";	
	#delimit cr;
	

	*** Variable changes
	rename 		 D_loan_now_EL D_loan_now1_EL
	rename 		 D_loan_all_BL D_loan_now1_BL // D_loan_now_BL is not consistent
	
	gen      	 attrition_main_EL   = (final_status_main != "COMPLETED")
	gen      	 attrition_spouse_EL = (final_status_spouse != "COMPLETED") 
	
	gen 		 win95_sp_h_labor_1_16_1_BL = 0 // no values at baseline
	gen 		 win95_sp_h_labor_1_17_1_BL = 0
	gen 		 win95_sp_h_labor_1_20_1_BL = 0
	gen 		 IND_marriage_sp_BL 		= 0
	gen 		 cognempathy_sjt_z_BL 		= 0
	gen 		 express_sjt_z_BL	 		= 0
	gen 		 influence_z_BL 			= 0
	gen 		 negotiate_z_BL 			= 0
	gen 		 IND_norms_sjt_BL 			= 0
	gen 		 intra_1_new10_z_BL 		= 0
	gen 		 intra_1_new11_z_BL 		= 0
	gen 		 intra_1_new12_z_BL 		= 0
	gen 		 intra_1_new13_z_BL 		= 0
	gen 		 D_survival_BL      		= 0 
	gen 		 win95_savings_BL   		= 0
	gen 		 attrition_main_BL   		= 0
	gen 		 attrition_spouse_BL   		= 0
	
	gen 		 treat1 = (treatment_new == 1) // treatment outcomes
	gen 		 treat2 = (treatment_new == 2)
	gen 		 T1_profit = treat1*D_profit_BL
	gen 		 T2_profit = treat2*D_profit_BL
	gen 		 T1_actual = (complete_HS == 1 & treatment_new == 1)
	gen 		 T2_actual = (complete_HS == 1 & treatment_new == 2)
	gen 		 T1act_profit = T1_actual*D_profit_BL
	gen 		 T2act_profit = T2_actual*D_profit_BL
	
	* Expressing monetary outcomes in Thousands Ethiopian Birr
	foreach surv in BL EL {
		foreach v in win95_sp_h_labor_1_16_1 win95_sp_h_labor_1_17_1 win95_sp_h_labor_1_20_1 ///
					 win95_b_bus_1_25 win95_b_bus_1_26 win95_b_bus_1_39 ///
					 win95_b_bus_1_27 win95_b_bus_1_28 win95_loan_size /// 
					 win95_savings {
				 cap replace `v'_`surv' = `v'_`surv'/1000
		}
	}
	
	
	*** Shorter labels for tables
	label var 	 treat1 "T1 - Scale up training"
	label var 	 treat2 "T2 - Scale up + couples training"
	label var 	 T1_profit "T1 * Above median profits"
	label var 	 T2_profit "T2 * Above median profits"
	label var 	 T1_actual "Full compliance in T1 training"
	label var 	 T2_actual "Full compliance in T2 training"
	label var 	 T1act_profit "T1 * Above median profits"
	label var 	 T2act_profit "T2 * Above median profits"
	label var 	 attrition_main_EL "Attrition (Female only)"
	label var 	 attrition_spouse_EL "Attrition (Spouse only)"
	label var 	 win95_sp_h_labor_1_16_1_EL "Husband's monthly revenues (Thousand ETB)"
	label var 	 win95_sp_h_labor_1_17_1_EL "Husband's monthly profits (Thousand ETB)"
	label var 	 win95_sp_h_labor_1_20_1_EL "Husband's capital investment (Thousand ETB)"
	label var 	 IND_marriage_sp_EL "Husband's marriage quality (Standardized index)"
	label var 	 win95_b_bus_1_25_EL "Monthly revenues (Thousand ETB)"
	label var 	 win95_b_bus_1_26_EL "Monthly profits (Thousand ETB)"
	label var 	 win95_b_bus_1_39_EL "Monthly costs (Thousand ETB)"
	label var 	 cognempathy_sjt_z_EL "Cognitive empathy (Z-score)"
	label var 	 express_sjt_z_EL "Expressiveness (Z-score)"
	label var 	 influence_EL "Interpersonal influence (Z-score)"
	label var 	 negotiate_EL "Negotiation (Z-score)"
	label var 	 IND_norms_sjt_EL "Perceived gender norms (Standardized index)"
	label var 	 intra_1_new10_z_EL ///
				 "Total women respondent believes would negotiate with their husband to do chores (Z-score)"
	label var 	 intra_1_new11_z_EL ///
				 "Total women respondent believes would speak badly of a woman whose husband did chores (Z-score)"
	label var 	 intra_1_new12_z_EL ///
				 "Likelihood of respondent to negotiate with her husband to do chores (Z-score)"
	label var 	 intra_1_new13_z_EL ///
				 "Likelihood of respondent to stop working away from home due to negative rumors (Z-score)"
	label var 	 D_survival_EL "Business survival (Yes = 1)"
	label var 	 win95_b_bus_1_27_EL "Annual revenues (Thousand ETB)"
	label var 	 win95_b_bus_1_28_EL "Annual profits (Thousand ETB)"
	label var 	 win95_employees_EL "Number of employees (Count)"
	label var 	 D_loan_now1_EL "Currently has a loan (Yes = 1)"
	label var 	 D_altfinance_EL "Borrowed informally (Yes = 1)"
	label var 	 win95_loan_size_EL "Total loans owed (Thousand ETB)"
	label var  	 win95_savings_EL "Accumulated savings (Thousand ETB)"
	

**#  2. Program for Regression Tables
* -----------------------------------------------------------------------------*
	program drop _all

	program define maketable, rclass
		// Accept inputs
		syntax , Outcomes(namelist) Treatvars(varlist) Model(string) Fixed(string)

		// Count total outcomes
		local n_outcomes : word count `outcomes'
	
		// Loop over outcomes
		forvalues i = 1/`n_outcomes' {
			local y : word `i' of `outcomes'
			qui reg `y'_EL `treatvars' `y'_BL $controls `fixed' , robust
			eststo `model'_`i'
			
			* 1. Control mean
			if "`model'" == "ate" {
				qui summarize `y'_EL if treat1 == 0 & treat2 == 0
				qui return list 
			}
			if "`model'" == "diff" {
				qui summarize `y'_EL if treat1 == 0 & treat2 == 0 & D_profit_BL == 0
				qui return list 
			}
			estadd scalar control_mean = r(mean): `model'_`i'
			
			* 2. Equality of treatment
			qui lincom treat1 - treat2
			local pval0 = r(p)
			estadd scalar pval0 = `pval0': `model'_`i'
			
			if "`model'" == "diff" {
				
				* 3. Marginal effect of T1 on low profits firms
				qui lincom treat1 + T1_profit
				local Est1 = r(estimate)
				local pval1 = r(p)
				estadd scalar Est1  = `Est1' : `model'_`i'
				estadd scalar pval1 = `pval1': `model'_`i'
				
				* 4. Marginal effect of T2 on low profits firms
				qui lincom treat2 + T2_profit
				local Est2 = r(estimate)
				local pval2 = r(p)
				estadd scalar Est2  = `Est2' : `model'_`i'
				estadd scalar pval2 = `pval2': `model'_`i'
			}
			
			* 5. Model type
			qui summarize `y'_BL
			qui return list
			if `r(mean)' == 0 {
				estadd local ancova = "" : `model'_`i'
			}
			else {
				estadd local ancova = "\checkmark" : `model'_`i'
			}
		}

end

	program define makelate, rclass
		// Accept inputs
		syntax , Outcomes(namelist) Treatvars(varlist) Latevars(varlist) Model(string) Fixed(string)

		// Count total outcomes
		local n_outcomes : word count `outcomes'
	
		// Loop over outcomes
		forvalues i = 1/`n_outcomes' {
			local y : word `i' of `outcomes'
			qui ivregress 2sls `y'_EL `y'_BL `fixed' $controls ///
			(`latevars' = `treatvars'), robust
			eststo `model'_`i'
			
			* 1. Control mean
			if "`model'" == "ate" {
				qui summarize `y'_EL if T1_actual == 0 & T2_actual == 0
				qui return list 
			}
			if "`model'" == "diff" {
				qui summarize `y'_EL if T1_actual == 0 & T2_actual == 0 & D_profit_BL == 0
				qui return list 
			}
			estadd scalar control_mean = r(mean): `model'_`i'
			
			* 2. Equality of treatment
			qui lincom T1_actual - T2_actual
			local pval0 = r(p)
			estadd scalar pval0 = `pval0': `model'_`i'
			
			if "`model'" == "diff" {
				
				* 3. Marginal effect of T1 on low profits firms
				qui lincom T1_actual + T1act_profit
				local Est1 = r(estimate)
				local pval1 = r(p)
				estadd scalar Est1  = `Est1' : `model'_`i'
				estadd scalar pval1 = `pval1': `model'_`i'
				
				* 4. Marginal effect of T2 on low profits firms
				qui lincom T2_actual + T2act_profit
				local Est2 = r(estimate)
				local pval2 = r(p)
				estadd scalar Est2  = `Est2' : `model'_`i'
				estadd scalar pval2 = `pval2': `model'_`i'
			}
			
			* 5. Model type
			qui summarize `y'_BL
			qui return list
			if `r(mean)' == 0 {
				estadd local ancova = "" : `model'_`i'
			}
			else {
				estadd local ancova = "\checkmark" : `model'_`i'
			}
		}

end


**#  TABLE A3. Attrition
* -----------------------------------------------------------------------------*
	eststo clear
	maketable, ///
		outcomes($attrition ) fixed(" ") ///
		treatvars(treat1 treat2) model(ate)
	maketable, ///
		outcomes($attrition ) fixed(" ") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)
	
	local tit "Correlation between Survey Attrition and Treatment Status"
	local lab "tab:App_attrition"
	
	forvalues i = 1/2 {
		est restore ate_`i' // collect labels for each model
		local depvar = e(depvar)
		local depvar_label : variable label `depvar'
		if "`depvar_label'" == "" local depvar_label "`depvar'"
		local mg_labels_`i' "`depvar_label'"
	}

	* Latex
	esttab ate_1 diff_1 ate_2 diff_2 ///
		using "$tables_app/TabA3_attrition", replace booktabs ///
		drop(attrition_main_BL attrition_spouse_BL) ///
        star(* 0.1 ** 0.05 *** 0.01) label mlabels(none) collabels(none) eqlabels(none) ///
		mgroups("`mg_labels_1'" "`mg_labels_2'", pattern(1 0 1 0) ///
		span erepeat(\cmidrule(l){@span}) prefix(\multicolumn{@span}{c}{) suffix(})) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2))) ///
		stats(N r2_a control_mean, fmt(0 %9.2f %15.2f) labels("Observations" "$r2" "Control mean")) nonote ///
		prehead("\begin{table}[!htbp] \begin{adjustbox}{max width=\textwidth} \begin{threeparttable} \caption{`tit'} \label{`lab'} \begin{tabular}{l*{@M}{c}} \toprule \toprule") ///
		postfoot(\bottomrule \end{tabular} \begin{tablenotes} \footnotesize $foot0 $foot1att $foot2att ///
		\end{tablenotes} \end{threeparttable} \end{adjustbox} \end{table})
		
	* Excel
	esttab ate_1 diff_1 ate_2 diff_2 ///
		using "$tables_app/TabA3_attrition", replace csv ///
		drop(attrition_main_BL attrition_spouse_BL) ///
        star(* 0.1 ** 0.05 *** 0.01) label collabels(none) eqlabels(none) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2))) ///
		stats(N r2 control_mean, fmt(0 %9.2f %15.2f) labels("Observations" "R-squared" "Control mean")) nonote
	
		
**#  TABLE A4. Husband Outcomes
* -----------------------------------------------------------------------------*		
	eststo clear
	maketable, ///
		outcomes($spouse ) fixed(" ") ///
		treatvars(treat1 treat2) model(ate)
	maketable, ///
		outcomes($spouse ) fixed(" ") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)
		
	local tit "Impact on Husband Business and Marriage Outcomes"
	local lab "tab:ATE_husband"
	
	forvalues i = 1/4 {
		est restore ate_`i'
		local depvar = e(depvar)
		local depvar_label : variable label `depvar'
		if "`depvar_label'" == "" local depvar_label "`depvar'"
		local mg_labels_`i' "`depvar_label'"
	}
	
	* Latex
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ///
		using "$tables_app/TabA4_husband", replace booktabs ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label mlabels(none) collabels(none) eqlabels(none) ///
		mgroups("`mg_labels_1'" "`mg_labels_2'" "`mg_labels_3'" "`mg_labels_4'", pattern(1 0 1 0 1 0 1 0) ///
		span erepeat(\cmidrule(l){@span}) prefix(\multicolumn{@span}{c}{) suffix(})) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2))) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f) ///
		labels("Observations" "$r2" "Control mean" "$pval0" "$Est1" "$pval1" "$Est2" "$pval2")) nonote ///
		prehead("\begin{table}[!htbp] \begin{adjustbox}{max width=\textwidth} \begin{threeparttable} \caption{`tit'} \label{`lab'} \begin{tabular}{l*{@M}{c}} \toprule \toprule") ///
		postfoot(\bottomrule \end{tabular} \begin{tablenotes} \footnotesize $foot0 $foot1sp $foot2 $foot3_8 ///
		\end{tablenotes} \end{threeparttable} \end{adjustbox} \end{table})
		
	* Excel
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ///
		using "$tables_app/TabA4_husband", replace csv ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label collabels(none) eqlabels(none) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2))) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f) ///
		labels("Observations" "Adj. R-squared" "Control mean" "$pval0_1" "$Est1_1" "$pval1_1" "$Est2_1" "$pval2_1")) nonote
		
		
**#  TABLE A5. Monthly Business Performance
* -----------------------------------------------------------------------------*		
	eststo clear
	maketable, ///
		outcomes($business2 ) fixed(" ") ///
		treatvars(treat1 treat2) model(ate)
	maketable, ///
		outcomes($business2 ) fixed(" ") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)
		
	local tit "Impact on Monthly-Recall Business Performance Outcomes"
	local lab "tab:App_business"
	
	forvalues i = 1/3 {
		est restore ate_`i'
		local depvar = e(depvar)
		local depvar_label : variable label `depvar'
		if "`depvar_label'" == "" local depvar_label "`depvar'"
		local mg_labels_`i' "`depvar_label'"
	}
	
	* Latex
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ///
		using "$tables_app/TabA5_business", replace booktabs ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label mlabels(none) collabels(none) eqlabels(none) ///
		mgroups("`mg_labels_1'" "`mg_labels_2'" "`mg_labels_3'", pattern(1 0 1 0 1 0) ///
		span erepeat(\cmidrule(l){@span}) prefix(\multicolumn{@span}{c}{) suffix(})) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2))) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2 ancova, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f 0) ///
		labels("Observations" "$r2" "Control mean" "$pval0" "$Est1" "$pval1" "$Est2" "$pval2" "ANCOVA model")) nonote ///
		prehead("\begin{table}[!htbp] \begin{adjustbox}{max width=\textwidth} \begin{threeparttable} \caption{`tit'} \label{`lab'} \begin{tabular}{l*{@M}{c}} \toprule \toprule") ///
		postfoot(\bottomrule \end{tabular} \begin{tablenotes} \footnotesize $foot0 $foot1bus2 $foot2 $foot3_6 ///
		\end{tablenotes} \end{threeparttable} \end{adjustbox} \end{table})
		
	* Excel
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ///
		using "$tables_app/TabA5_business", replace csv ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label collabels(none) eqlabels(none) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2))) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f) ///
		labels("Observations" "Adj. R-squared" "Control mean" "$pval0_1" "$Est1_1" "$pval1_1" "$Est2_1" "$pval2_1")) nonote
		
		
**#  TABLE A6. Interpersonal Skills
* -----------------------------------------------------------------------------*		
	eststo clear
	
	* Make balanced sample
	qui reg treatment_new selfesteem_EL selfefficacy_EL persgrowth_EL proactivity_EL ///
				cognempathy_sjt_EL express_sjt_EL influence_EL negotiate_EL, robust
	gen reg_samp = 1 if e(sample)
	
	maketable, ///
		outcomes($ses2 ) fixed("if reg_samp == 1") ///
		treatvars(treat1 treat2) model(ate)
	maketable, ///
		outcomes($ses2 ) fixed("if reg_samp == 1") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)
		
	local tit "Impact on Interpersonal Skills Sub-Components"
	local lab "tab:App_ses"
	
	forvalues i = 1/4 {
		est restore ate_`i'
		local depvar = e(depvar)
		local depvar_label : variable label `depvar'
		if "`depvar_label'" == "" local depvar_label "`depvar'"
		local mg_labels_`i' "`depvar_label'"
	}
	
	* Latex
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ///
		using "$tables_app/TabA6_ses", replace booktabs ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label mlabels(none) collabels(none) eqlabels(none) ///
		mgroups("`mg_labels_1'" "`mg_labels_2'" "`mg_labels_3'" "`mg_labels_4'", pattern(1 0 1 0 1 0 1 0) ///
		span erepeat(\cmidrule(l){@span}) prefix(\multicolumn{@span}{c}{) suffix(})) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2))) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f) ///
		labels("Observations" "$r2" "Control mean" "$pval0" "$Est1" "$pval1" "$Est2" "$pval2")) nonote ///
		prehead("\begin{table}[!htbp] \begin{adjustbox}{max width=\textwidth} \begin{threeparttable} \caption{`tit'} \label{`lab'} \begin{tabular}{l*{@M}{c}} \toprule \toprule") ///
		postfoot(\bottomrule \end{tabular} \begin{tablenotes} \footnotesize $foot0 $foot1ses $foot2 $foot3_8 ///
		\end{tablenotes} \end{threeparttable} \end{adjustbox} \end{table})
		
	* Excel
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ///
		using "$tables_app/TabA6_ses", replace csv ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label collabels(none) eqlabels(none) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2))) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f) ///
		labels("Observations" "Adj. R-squared" "Control mean" "$pval0_1" "$Est1_1" "$pval1_1" "$Est2_1" "$pval2_1")) nonote
		
		
**#  TABLE A7. Gender Norms
* -----------------------------------------------------------------------------*
	eststo clear
	
	* Make balanced sample
	drop reg_samp
	qui reg treatment_new intra_1_new10_z_EL intra_1_new11_z_EL intra_1_new12_z_EL ///
							intra_1_new13_z_EL, robust
	gen reg_samp = 1 if e(sample)
	
	maketable, ///
		outcomes($norms ) fixed("if reg_samp == 1") ///
		treatvars(treat1 treat2) model(ate)
	maketable, ///
		outcomes($norms ) fixed("if reg_samp == 1") ///
		treatvars(treat1 treat2 D_profit_BL T1_profit T2_profit) model(diff)
		
	local tit "Impact on Perceived Gender Norms"
	local lab "tab:App_norms"
	
	forvalues i = 1/5 {
		est restore ate_`i'
		local depvar = e(depvar)
		local depvar_label : variable label `depvar'
		if "`depvar_label'" == "" local depvar_label "`depvar'"
		local mg_labels_`i' "`depvar_label'"
	}
	
	* Latex
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ate_5 diff_5 ///
		using "$tables_app/TabA7_norms", replace booktabs ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label mlabels(none) collabels(none) eqlabels(none) ///
		mgroups("`mg_labels_1'" "`mg_labels_2'" "`mg_labels_3'" "`mg_labels_4'" "`mg_labels_5'", pattern(1 0 1 0 1 0 1 0 1 0) ///
		span erepeat(\cmidrule(l){@span}) prefix(\multicolumn{@span}{c}{) suffix(})) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2))) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f) ///
		labels("Observations" "$r2" "Control mean" "$pval0" "$Est1" "$pval1" "$Est2" "$pval2")) nonote ///
		prehead("\begin{table}[!htbp] \begin{adjustbox}{max width=\textwidth} \begin{threeparttable} \caption{`tit'} \label{`lab'} \begin{tabular}{l*{@M}{c}} \toprule \toprule") ///
		postfoot(\bottomrule \end{tabular} \begin{tablenotes} \footnotesize $foot0 $foot1norm $foot2 $foot3_10 ///
		\end{tablenotes} \end{threeparttable} \end{adjustbox} \end{table})
		
	* Excel
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ate_5 diff_5 ///
		using "$tables_app/TabA7_norms", replace csv ///
		keep(treat1 treat2 D_profit_BL T1_profit T2_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label collabels(none) eqlabels(none) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2))) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f) ///
		labels("Observations" "Adj. R-squared" "Control mean" "$pval0_1" "$Est1_1" "$pval1_1" "$Est2_1" "$pval2_1")) nonote


**#  TABLE A8. LATE - Business Performance
* -----------------------------------------------------------------------------*
	eststo clear
	makelate, ///
		outcomes($business ) latevars(T1_actual T2_actual) ///
		treatvars(treat1 treat2) model(ate) fixed(" ")
	makelate, ///
		outcomes($business ) ///
		treatvars(treat1 treat2 T1_profit T2_profit) fixed(D_profit_BL) ///
		latevars(T1_actual T2_actual T1act_profit T2act_profit) model(diff)
		
	local tit "Impact on Business Performance (LATE Estimator)"
	local lab "tab:App_bus_LATE"
	
	forvalues i = 1/4 {
		est restore ate_`i'
		local depvar = e(depvar)
		local depvar_label : variable label `depvar'
		if "`depvar_label'" == "" local depvar_label "`depvar'"
		local mg_labels_`i' "`depvar_label'"
	}
	
	* Latex
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ///
		using "$tables_app/TabA8_business_late", replace booktabs ///
		keep(T1_actual T2_actual D_profit_BL T1act_profit T2act_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label mlabels(none) collabels(none) eqlabels(none) ///
		mgroups("`mg_labels_1'" "`mg_labels_2'" "`mg_labels_3'" "`mg_labels_4'", pattern(1 0 1 0 1 0 1 0) ///
		span erepeat(\cmidrule(l){@span}) prefix(\multicolumn{@span}{c}{) suffix(})) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2))) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2 ancova, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f 0) ///
		labels("Observations" "$r2" "Control mean" "$pval0" "$Est1" "$pval1" "$Est2" "$pval2" "ANCOVA model")) nonote ///
		prehead("\begin{table}[!htbp] \begin{adjustbox}{max width=\textwidth} \begin{threeparttable} \caption{`tit'} \label{`lab'} \begin{tabular}{l*{@M}{c}} \toprule \toprule") ///
		postfoot(\bottomrule \end{tabular} \begin{tablenotes} \footnotesize $foot0 $foot1bus $foot2 $foot3_8 ///
		\end{tablenotes} \end{threeparttable} \end{adjustbox} \end{table})
		
	* Excel
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ///
		using "$tables_app/TabA8_business_late", replace csv ///
		keep(T1_actual T2_actual D_profit_BL T1act_profit T2act_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label collabels(none) eqlabels(none) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2))) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f) ///
		labels("Observations" "Adj. R-squared" "Control mean" "$pval0_1" "$Est1_1" "$pval1_1" "$Est2_1" "$pval2_1")) nonote


**#  TABLE A9. LATE - Financial Outcomes
* -----------------------------------------------------------------------------*
	eststo clear
	makelate, ///
		outcomes($finance ) ///
		treatvars(treat1 treat2) latevars(T1_actual T2_actual) ///
		model(ate) fixed(" ")
	makelate, ///
		outcomes($finance ) ///
		treatvars(treat1 treat2 T1_profit T2_profit) fixed(D_profit_BL) ///
		latevars(T1_actual T2_actual T1act_profit T2act_profit) model(diff)
	
	local tit "Impact on Financial Outcomes (LATE Estimator)"
	local lab "tab:App_fin_LATE"
	
	forvalues i = 1/4 {
		est restore ate_`i'
		local depvar = e(depvar)
		local depvar_label : variable label `depvar'
		if "`depvar_label'" == "" local depvar_label "`depvar'"
		local mg_labels_`i' "`depvar_label'"
	}
	
	* Latex
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ///
		using "$tables_app/TabA9_finance_late", replace booktabs ///
		keep(T1_actual T2_actual D_profit_BL T1act_profit T2act_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label mlabels(none) collabels(none) eqlabels(none) ///
		mgroups("`mg_labels_1'" "`mg_labels_2'" "`mg_labels_3'" "`mg_labels_4'", pattern(1 0 1 0 1 0 1 0) ///
		span erepeat(\cmidrule(l){@span}) prefix(\multicolumn{@span}{c}{) suffix(})) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2))) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2 ancova, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f 0) ///
		labels("Observations" "$r2" "Control mean" "$pval0" "$Est1" "$pval1" "$Est2" "$pval2" "ANCOVA model")) nonote ///
		prehead("\begin{table}[!htbp] \begin{adjustbox}{max width=\textwidth} \begin{threeparttable} \caption{`tit'} \label{`lab'} \begin{tabular}{l*{@M}{c}} \toprule \toprule") ///
		postfoot(\bottomrule \end{tabular} \begin{tablenotes} \footnotesize $foot0 $foot1fin $foot2 $foot3_8 ///
		\end{tablenotes} \end{threeparttable} \end{adjustbox} \end{table})
		
	* Excel
	esttab ate_1 diff_1 ate_2 diff_2 ate_3 diff_3 ate_4 diff_4 ///
		using "$tables_app/TabA9_finance_late", replace csv ///
		keep(T1_actual T2_actual D_profit_BL T1act_profit T2act_profit) ///
        star(* 0.1 ** 0.05 *** 0.01) label collabels(none) eqlabels(none) ///
        cells(b(fmt(2) pvalue(p) star) se(par fmt(2))) ///
		stats(N r2_a control_mean pval0 Est1 pval1 Est2 pval2, fmt(0 %9.2f %15.2f %9.2f %15.2f %9.2f %15.2f %9.2f) ///
		labels("Observations" "Adj. R-squared" "Control mean" "$pval0_1" "$Est1_1" "$pval1_1" "$Est2_1" "$pval2_1")) nonote
		
		
********************************************************************************
*                          END OF ANALYSIS DOFILE                              *
********************************************************************************
